#ifndef OPENGL_VIRT
#define OPENGL_VIRT
typedef struct OpenGLVTab
	{
#define VFUNC(type,name,mem,args) type (*mem) args;
#define VVAR(type,name,mem)       type (*mem);
#include "OpenGL.vf"
#undef VFUNC
#undef VVAR
} OpenGLVTab;
extern OpenGLVTab *OpenGLVPtr;
#define D_OPENGL OpenGLVTab *OpenGLVPtr
#define I_OPENGL OpenGLVPtr = (OpenGLVTab *) SvIV(perl_get_sv("PDL::Graphics::OpenGLVPtr",5));   
#ifndef OPENGL_NOVIRT
#define glAccum (*OpenGLVPtr->V_glAccum)
#define glAlphaFunc (*OpenGLVPtr->V_glAlphaFunc)
#define glBegin (*OpenGLVPtr->V_glBegin)
#define glBitmap (*OpenGLVPtr->V_glBitmap)
#define glBlendFunc (*OpenGLVPtr->V_glBlendFunc)
#define glCallList (*OpenGLVPtr->V_glCallList)
#define glCallLists (*OpenGLVPtr->V_glCallLists)
#define glClear (*OpenGLVPtr->V_glClear)
#define glClearAccum (*OpenGLVPtr->V_glClearAccum)
#define glClearColor (*OpenGLVPtr->V_glClearColor)
#define glClearDepth (*OpenGLVPtr->V_glClearDepth)
#define glClearIndex (*OpenGLVPtr->V_glClearIndex)
#define glClearStencil (*OpenGLVPtr->V_glClearStencil)
#define glClipPlane (*OpenGLVPtr->V_glClipPlane)
#define glColor3b (*OpenGLVPtr->V_glColor3b)
#define glColor3bv (*OpenGLVPtr->V_glColor3bv)
#define glColor3d (*OpenGLVPtr->V_glColor3d)
#define glColor3dv (*OpenGLVPtr->V_glColor3dv)
#define glColor3f (*OpenGLVPtr->V_glColor3f)
#define glColor3fv (*OpenGLVPtr->V_glColor3fv)
#define glColor3i (*OpenGLVPtr->V_glColor3i)
#define glColor3iv (*OpenGLVPtr->V_glColor3iv)
#define glColor3s (*OpenGLVPtr->V_glColor3s)
#define glColor3sv (*OpenGLVPtr->V_glColor3sv)
#define glColor3ub (*OpenGLVPtr->V_glColor3ub)
#define glColor3ubv (*OpenGLVPtr->V_glColor3ubv)
#define glColor3ui (*OpenGLVPtr->V_glColor3ui)
#define glColor3uiv (*OpenGLVPtr->V_glColor3uiv)
#define glColor3us (*OpenGLVPtr->V_glColor3us)
#define glColor3usv (*OpenGLVPtr->V_glColor3usv)
#define glColor4b (*OpenGLVPtr->V_glColor4b)
#define glColor4bv (*OpenGLVPtr->V_glColor4bv)
#define glColor4d (*OpenGLVPtr->V_glColor4d)
#define glColor4dv (*OpenGLVPtr->V_glColor4dv)
#define glColor4f (*OpenGLVPtr->V_glColor4f)
#define glColor4fv (*OpenGLVPtr->V_glColor4fv)
#define glColor4i (*OpenGLVPtr->V_glColor4i)
#define glColor4iv (*OpenGLVPtr->V_glColor4iv)
#define glColor4s (*OpenGLVPtr->V_glColor4s)
#define glColor4sv (*OpenGLVPtr->V_glColor4sv)
#define glColor4ub (*OpenGLVPtr->V_glColor4ub)
#define glColor4ubv (*OpenGLVPtr->V_glColor4ubv)
#define glColor4ui (*OpenGLVPtr->V_glColor4ui)
#define glColor4uiv (*OpenGLVPtr->V_glColor4uiv)
#define glColor4us (*OpenGLVPtr->V_glColor4us)
#define glColor4usv (*OpenGLVPtr->V_glColor4usv)
#define glColorMask (*OpenGLVPtr->V_glColorMask)
#define glColorMaterial (*OpenGLVPtr->V_glColorMaterial)
#define glCopyPixels (*OpenGLVPtr->V_glCopyPixels)
#define glCullFace (*OpenGLVPtr->V_glCullFace)
#define glDeleteLists (*OpenGLVPtr->V_glDeleteLists)
#define glDepthFunc (*OpenGLVPtr->V_glDepthFunc)
#define glDepthMask (*OpenGLVPtr->V_glDepthMask)
#define glDepthRange (*OpenGLVPtr->V_glDepthRange)
#define glDisable (*OpenGLVPtr->V_glDisable)
#define glDrawBuffer (*OpenGLVPtr->V_glDrawBuffer)
#define glDrawPixels (*OpenGLVPtr->V_glDrawPixels)
#define glEdgeFlag (*OpenGLVPtr->V_glEdgeFlag)
#define glEdgeFlagv (*OpenGLVPtr->V_glEdgeFlagv)
#define glEnable (*OpenGLVPtr->V_glEnable)
#define glEnd (*OpenGLVPtr->V_glEnd)
#define glEndList (*OpenGLVPtr->V_glEndList)
#define glEvalCoord1d (*OpenGLVPtr->V_glEvalCoord1d)
#define glEvalCoord1dv (*OpenGLVPtr->V_glEvalCoord1dv)
#define glEvalCoord1f (*OpenGLVPtr->V_glEvalCoord1f)
#define glEvalCoord1fv (*OpenGLVPtr->V_glEvalCoord1fv)
#define glEvalCoord2d (*OpenGLVPtr->V_glEvalCoord2d)
#define glEvalCoord2dv (*OpenGLVPtr->V_glEvalCoord2dv)
#define glEvalCoord2f (*OpenGLVPtr->V_glEvalCoord2f)
#define glEvalCoord2fv (*OpenGLVPtr->V_glEvalCoord2fv)
#define glEvalMesh1 (*OpenGLVPtr->V_glEvalMesh1)
#define glEvalMesh2 (*OpenGLVPtr->V_glEvalMesh2)
#define glEvalPoint1 (*OpenGLVPtr->V_glEvalPoint1)
#define glEvalPoint2 (*OpenGLVPtr->V_glEvalPoint2)
#define glFeedbackBuffer (*OpenGLVPtr->V_glFeedbackBuffer)
#define glFinish (*OpenGLVPtr->V_glFinish)
#define glFlush (*OpenGLVPtr->V_glFlush)
#define glFogf (*OpenGLVPtr->V_glFogf)
#define glFogfv (*OpenGLVPtr->V_glFogfv)
#define glFogi (*OpenGLVPtr->V_glFogi)
#define glFogiv (*OpenGLVPtr->V_glFogiv)
#define glFrontFace (*OpenGLVPtr->V_glFrontFace)
#define glFrustum (*OpenGLVPtr->V_glFrustum)
#define glGenLists (*OpenGLVPtr->V_glGenLists)
#define glGetBooleanv (*OpenGLVPtr->V_glGetBooleanv)
#define glGetClipPlane (*OpenGLVPtr->V_glGetClipPlane)
#define glGetDoublev (*OpenGLVPtr->V_glGetDoublev)
#define glGetError (*OpenGLVPtr->V_glGetError)
#define glGetFloatv (*OpenGLVPtr->V_glGetFloatv)
#define glGetIntegerv (*OpenGLVPtr->V_glGetIntegerv)
#define glGetLightfv (*OpenGLVPtr->V_glGetLightfv)
#define glGetLightiv (*OpenGLVPtr->V_glGetLightiv)
#define glGetMapdv (*OpenGLVPtr->V_glGetMapdv)
#define glGetMapfv (*OpenGLVPtr->V_glGetMapfv)
#define glGetMapiv (*OpenGLVPtr->V_glGetMapiv)
#define glGetMaterialfv (*OpenGLVPtr->V_glGetMaterialfv)
#define glGetMaterialiv (*OpenGLVPtr->V_glGetMaterialiv)
#define glGetPixelMapfv (*OpenGLVPtr->V_glGetPixelMapfv)
#define glGetPixelMapuiv (*OpenGLVPtr->V_glGetPixelMapuiv)
#define glGetPixelMapusv (*OpenGLVPtr->V_glGetPixelMapusv)
#define glGetPolygonStipple (*OpenGLVPtr->V_glGetPolygonStipple)
#define glGetTexEnvfv (*OpenGLVPtr->V_glGetTexEnvfv)
#define glGetTexEnviv (*OpenGLVPtr->V_glGetTexEnviv)
#define glGetTexGendv (*OpenGLVPtr->V_glGetTexGendv)
#define glGetTexGenfv (*OpenGLVPtr->V_glGetTexGenfv)
#define glGetTexGeniv (*OpenGLVPtr->V_glGetTexGeniv)
#define glGetTexImage (*OpenGLVPtr->V_glGetTexImage)
#define glGetTexLevelParameterfv (*OpenGLVPtr->V_glGetTexLevelParameterfv)
#define glGetTexLevelParameteriv (*OpenGLVPtr->V_glGetTexLevelParameteriv)
#define glGetTexParameterfv (*OpenGLVPtr->V_glGetTexParameterfv)
#define glGetTexParameteriv (*OpenGLVPtr->V_glGetTexParameteriv)
#define glHint (*OpenGLVPtr->V_glHint)
#define glIndexMask (*OpenGLVPtr->V_glIndexMask)
#define glIndexd (*OpenGLVPtr->V_glIndexd)
#define glIndexdv (*OpenGLVPtr->V_glIndexdv)
#define glIndexf (*OpenGLVPtr->V_glIndexf)
#define glIndexfv (*OpenGLVPtr->V_glIndexfv)
#define glIndexi (*OpenGLVPtr->V_glIndexi)
#define glIndexiv (*OpenGLVPtr->V_glIndexiv)
#define glIndexs (*OpenGLVPtr->V_glIndexs)
#define glIndexsv (*OpenGLVPtr->V_glIndexsv)
#define glInitNames (*OpenGLVPtr->V_glInitNames)
#define glIsEnabled (*OpenGLVPtr->V_glIsEnabled)
#define glIsList (*OpenGLVPtr->V_glIsList)
#define glLightModelf (*OpenGLVPtr->V_glLightModelf)
#define glLightModelfv (*OpenGLVPtr->V_glLightModelfv)
#define glLightModeli (*OpenGLVPtr->V_glLightModeli)
#define glLightModeliv (*OpenGLVPtr->V_glLightModeliv)
#define glLightf (*OpenGLVPtr->V_glLightf)
#define glLightfv (*OpenGLVPtr->V_glLightfv)
#define glLighti (*OpenGLVPtr->V_glLighti)
#define glLightiv (*OpenGLVPtr->V_glLightiv)
#define glLineStipple (*OpenGLVPtr->V_glLineStipple)
#define glLineWidth (*OpenGLVPtr->V_glLineWidth)
#define glListBase (*OpenGLVPtr->V_glListBase)
#define glLoadIdentity (*OpenGLVPtr->V_glLoadIdentity)
#define glLoadMatrixd (*OpenGLVPtr->V_glLoadMatrixd)
#define glLoadMatrixf (*OpenGLVPtr->V_glLoadMatrixf)
#define glLoadName (*OpenGLVPtr->V_glLoadName)
#define glLogicOp (*OpenGLVPtr->V_glLogicOp)
#define glMap1d (*OpenGLVPtr->V_glMap1d)
#define glMap1f (*OpenGLVPtr->V_glMap1f)
#define glMap2d (*OpenGLVPtr->V_glMap2d)
#define glMap2f (*OpenGLVPtr->V_glMap2f)
#define glMapGrid1d (*OpenGLVPtr->V_glMapGrid1d)
#define glMapGrid1f (*OpenGLVPtr->V_glMapGrid1f)
#define glMapGrid2d (*OpenGLVPtr->V_glMapGrid2d)
#define glMapGrid2f (*OpenGLVPtr->V_glMapGrid2f)
#define glMaterialf (*OpenGLVPtr->V_glMaterialf)
#define glMaterialfv (*OpenGLVPtr->V_glMaterialfv)
#define glMateriali (*OpenGLVPtr->V_glMateriali)
#define glMaterialiv (*OpenGLVPtr->V_glMaterialiv)
#define glMatrixMode (*OpenGLVPtr->V_glMatrixMode)
#define glMultMatrixd (*OpenGLVPtr->V_glMultMatrixd)
#define glMultMatrixf (*OpenGLVPtr->V_glMultMatrixf)
#define glNewList (*OpenGLVPtr->V_glNewList)
#define glNormal3b (*OpenGLVPtr->V_glNormal3b)
#define glNormal3bv (*OpenGLVPtr->V_glNormal3bv)
#define glNormal3d (*OpenGLVPtr->V_glNormal3d)
#define glNormal3dv (*OpenGLVPtr->V_glNormal3dv)
#define glNormal3f (*OpenGLVPtr->V_glNormal3f)
#define glNormal3fv (*OpenGLVPtr->V_glNormal3fv)
#define glNormal3i (*OpenGLVPtr->V_glNormal3i)
#define glNormal3iv (*OpenGLVPtr->V_glNormal3iv)
#define glNormal3s (*OpenGLVPtr->V_glNormal3s)
#define glNormal3sv (*OpenGLVPtr->V_glNormal3sv)
#define glOrtho (*OpenGLVPtr->V_glOrtho)
#define glPassThrough (*OpenGLVPtr->V_glPassThrough)
#define glPixelMapfv (*OpenGLVPtr->V_glPixelMapfv)
#define glPixelMapuiv (*OpenGLVPtr->V_glPixelMapuiv)
#define glPixelMapusv (*OpenGLVPtr->V_glPixelMapusv)
#define glPixelStoref (*OpenGLVPtr->V_glPixelStoref)
#define glPixelStorei (*OpenGLVPtr->V_glPixelStorei)
#define glPixelTransferf (*OpenGLVPtr->V_glPixelTransferf)
#define glPixelTransferi (*OpenGLVPtr->V_glPixelTransferi)
#define glPixelZoom (*OpenGLVPtr->V_glPixelZoom)
#define glPointSize (*OpenGLVPtr->V_glPointSize)
#define glPolygonMode (*OpenGLVPtr->V_glPolygonMode)
#define glPolygonStipple (*OpenGLVPtr->V_glPolygonStipple)
#define glPopAttrib (*OpenGLVPtr->V_glPopAttrib)
#define glPopMatrix (*OpenGLVPtr->V_glPopMatrix)
#define glPopName (*OpenGLVPtr->V_glPopName)
#define glPushAttrib (*OpenGLVPtr->V_glPushAttrib)
#define glPushMatrix (*OpenGLVPtr->V_glPushMatrix)
#define glPushName (*OpenGLVPtr->V_glPushName)
#define glRasterPos2d (*OpenGLVPtr->V_glRasterPos2d)
#define glRasterPos2dv (*OpenGLVPtr->V_glRasterPos2dv)
#define glRasterPos2f (*OpenGLVPtr->V_glRasterPos2f)
#define glRasterPos2fv (*OpenGLVPtr->V_glRasterPos2fv)
#define glRasterPos2i (*OpenGLVPtr->V_glRasterPos2i)
#define glRasterPos2iv (*OpenGLVPtr->V_glRasterPos2iv)
#define glRasterPos2s (*OpenGLVPtr->V_glRasterPos2s)
#define glRasterPos2sv (*OpenGLVPtr->V_glRasterPos2sv)
#define glRasterPos3d (*OpenGLVPtr->V_glRasterPos3d)
#define glRasterPos3dv (*OpenGLVPtr->V_glRasterPos3dv)
#define glRasterPos3f (*OpenGLVPtr->V_glRasterPos3f)
#define glRasterPos3fv (*OpenGLVPtr->V_glRasterPos3fv)
#define glRasterPos3i (*OpenGLVPtr->V_glRasterPos3i)
#define glRasterPos3iv (*OpenGLVPtr->V_glRasterPos3iv)
#define glRasterPos3s (*OpenGLVPtr->V_glRasterPos3s)
#define glRasterPos3sv (*OpenGLVPtr->V_glRasterPos3sv)
#define glRasterPos4d (*OpenGLVPtr->V_glRasterPos4d)
#define glRasterPos4dv (*OpenGLVPtr->V_glRasterPos4dv)
#define glRasterPos4f (*OpenGLVPtr->V_glRasterPos4f)
#define glRasterPos4fv (*OpenGLVPtr->V_glRasterPos4fv)
#define glRasterPos4i (*OpenGLVPtr->V_glRasterPos4i)
#define glRasterPos4iv (*OpenGLVPtr->V_glRasterPos4iv)
#define glRasterPos4s (*OpenGLVPtr->V_glRasterPos4s)
#define glRasterPos4sv (*OpenGLVPtr->V_glRasterPos4sv)
#define glReadBuffer (*OpenGLVPtr->V_glReadBuffer)
#define glReadPixels (*OpenGLVPtr->V_glReadPixels)
#define glRectd (*OpenGLVPtr->V_glRectd)
#define glRectdv (*OpenGLVPtr->V_glRectdv)
#define glRectf (*OpenGLVPtr->V_glRectf)
#define glRectfv (*OpenGLVPtr->V_glRectfv)
#define glRecti (*OpenGLVPtr->V_glRecti)
#define glRectiv (*OpenGLVPtr->V_glRectiv)
#define glRects (*OpenGLVPtr->V_glRects)
#define glRectsv (*OpenGLVPtr->V_glRectsv)
#define glRenderMode (*OpenGLVPtr->V_glRenderMode)
#define glRotated (*OpenGLVPtr->V_glRotated)
#define glRotatef (*OpenGLVPtr->V_glRotatef)
#define glScaled (*OpenGLVPtr->V_glScaled)
#define glScalef (*OpenGLVPtr->V_glScalef)
#define glScissor (*OpenGLVPtr->V_glScissor)
#define glSelectBuffer (*OpenGLVPtr->V_glSelectBuffer)
#define glShadeModel (*OpenGLVPtr->V_glShadeModel)
#define glStencilFunc (*OpenGLVPtr->V_glStencilFunc)
#define glStencilMask (*OpenGLVPtr->V_glStencilMask)
#define glStencilOp (*OpenGLVPtr->V_glStencilOp)
#define glTexCoord1d (*OpenGLVPtr->V_glTexCoord1d)
#define glTexCoord1dv (*OpenGLVPtr->V_glTexCoord1dv)
#define glTexCoord1f (*OpenGLVPtr->V_glTexCoord1f)
#define glTexCoord1fv (*OpenGLVPtr->V_glTexCoord1fv)
#define glTexCoord1i (*OpenGLVPtr->V_glTexCoord1i)
#define glTexCoord1iv (*OpenGLVPtr->V_glTexCoord1iv)
#define glTexCoord1s (*OpenGLVPtr->V_glTexCoord1s)
#define glTexCoord1sv (*OpenGLVPtr->V_glTexCoord1sv)
#define glTexCoord2d (*OpenGLVPtr->V_glTexCoord2d)
#define glTexCoord2dv (*OpenGLVPtr->V_glTexCoord2dv)
#define glTexCoord2f (*OpenGLVPtr->V_glTexCoord2f)
#define glTexCoord2fv (*OpenGLVPtr->V_glTexCoord2fv)
#define glTexCoord2i (*OpenGLVPtr->V_glTexCoord2i)
#define glTexCoord2iv (*OpenGLVPtr->V_glTexCoord2iv)
#define glTexCoord2s (*OpenGLVPtr->V_glTexCoord2s)
#define glTexCoord2sv (*OpenGLVPtr->V_glTexCoord2sv)
#define glTexCoord3d (*OpenGLVPtr->V_glTexCoord3d)
#define glTexCoord3dv (*OpenGLVPtr->V_glTexCoord3dv)
#define glTexCoord3f (*OpenGLVPtr->V_glTexCoord3f)
#define glTexCoord3fv (*OpenGLVPtr->V_glTexCoord3fv)
#define glTexCoord3i (*OpenGLVPtr->V_glTexCoord3i)
#define glTexCoord3iv (*OpenGLVPtr->V_glTexCoord3iv)
#define glTexCoord3s (*OpenGLVPtr->V_glTexCoord3s)
#define glTexCoord3sv (*OpenGLVPtr->V_glTexCoord3sv)
#define glTexCoord4d (*OpenGLVPtr->V_glTexCoord4d)
#define glTexCoord4dv (*OpenGLVPtr->V_glTexCoord4dv)
#define glTexCoord4f (*OpenGLVPtr->V_glTexCoord4f)
#define glTexCoord4fv (*OpenGLVPtr->V_glTexCoord4fv)
#define glTexCoord4i (*OpenGLVPtr->V_glTexCoord4i)
#define glTexCoord4iv (*OpenGLVPtr->V_glTexCoord4iv)
#define glTexCoord4s (*OpenGLVPtr->V_glTexCoord4s)
#define glTexCoord4sv (*OpenGLVPtr->V_glTexCoord4sv)
#define glTexEnvf (*OpenGLVPtr->V_glTexEnvf)
#define glTexEnvfv (*OpenGLVPtr->V_glTexEnvfv)
#define glTexEnvi (*OpenGLVPtr->V_glTexEnvi)
#define glTexEnviv (*OpenGLVPtr->V_glTexEnviv)
#define glTexGend (*OpenGLVPtr->V_glTexGend)
#define glTexGendv (*OpenGLVPtr->V_glTexGendv)
#define glTexGenf (*OpenGLVPtr->V_glTexGenf)
#define glTexGenfv (*OpenGLVPtr->V_glTexGenfv)
#define glTexGeni (*OpenGLVPtr->V_glTexGeni)
#define glTexGeniv (*OpenGLVPtr->V_glTexGeniv)
#define glTexImage1D (*OpenGLVPtr->V_glTexImage1D)
#define glTexImage2D (*OpenGLVPtr->V_glTexImage2D)
#define glTexParameterf (*OpenGLVPtr->V_glTexParameterf)
#define glTexParameterfv (*OpenGLVPtr->V_glTexParameterfv)
#define glTexParameteri (*OpenGLVPtr->V_glTexParameteri)
#define glTexParameteriv (*OpenGLVPtr->V_glTexParameteriv)
#define glTranslated (*OpenGLVPtr->V_glTranslated)
#define glTranslatef (*OpenGLVPtr->V_glTranslatef)
#define glVertex2d (*OpenGLVPtr->V_glVertex2d)
#define glVertex2dv (*OpenGLVPtr->V_glVertex2dv)
#define glVertex2f (*OpenGLVPtr->V_glVertex2f)
#define glVertex2fv (*OpenGLVPtr->V_glVertex2fv)
#define glVertex2i (*OpenGLVPtr->V_glVertex2i)
#define glVertex2iv (*OpenGLVPtr->V_glVertex2iv)
#define glVertex2s (*OpenGLVPtr->V_glVertex2s)
#define glVertex2sv (*OpenGLVPtr->V_glVertex2sv)
#define glVertex3d (*OpenGLVPtr->V_glVertex3d)
#define glVertex3dv (*OpenGLVPtr->V_glVertex3dv)
#define glVertex3f (*OpenGLVPtr->V_glVertex3f)
#define glVertex3fv (*OpenGLVPtr->V_glVertex3fv)
#define glVertex3i (*OpenGLVPtr->V_glVertex3i)
#define glVertex3iv (*OpenGLVPtr->V_glVertex3iv)
#define glVertex3s (*OpenGLVPtr->V_glVertex3s)
#define glVertex3sv (*OpenGLVPtr->V_glVertex3sv)
#define glVertex4d (*OpenGLVPtr->V_glVertex4d)
#define glVertex4dv (*OpenGLVPtr->V_glVertex4dv)
#define glVertex4f (*OpenGLVPtr->V_glVertex4f)
#define glVertex4fv (*OpenGLVPtr->V_glVertex4fv)
#define glVertex4i (*OpenGLVPtr->V_glVertex4i)
#define glVertex4iv (*OpenGLVPtr->V_glVertex4iv)
#define glVertex4s (*OpenGLVPtr->V_glVertex4s)
#define glVertex4sv (*OpenGLVPtr->V_glVertex4sv)
#define glViewport (*OpenGLVPtr->V_glViewport)
#define glAreTexturesResidentEXT (*OpenGLVPtr->V_glAreTexturesResidentEXT)
#define glArrayElementEXT (*OpenGLVPtr->V_glArrayElementEXT)
#define glBindTextureEXT (*OpenGLVPtr->V_glBindTextureEXT)
#define glColorPointerEXT (*OpenGLVPtr->V_glColorPointerEXT)
#define glDeleteTexturesEXT (*OpenGLVPtr->V_glDeleteTexturesEXT)
#define glDrawArraysEXT (*OpenGLVPtr->V_glDrawArraysEXT)
#define glEdgeFlagPointerEXT (*OpenGLVPtr->V_glEdgeFlagPointerEXT)
#define glGenTexturesEXT (*OpenGLVPtr->V_glGenTexturesEXT)
#define glIndexPointerEXT (*OpenGLVPtr->V_glIndexPointerEXT)
#define glIsTextureEXT (*OpenGLVPtr->V_glIsTextureEXT)
#define glNormalPointerEXT (*OpenGLVPtr->V_glNormalPointerEXT)
#define glPrioritizeTexturesEXT (*OpenGLVPtr->V_glPrioritizeTexturesEXT)
#define glTexCoordPointerEXT (*OpenGLVPtr->V_glTexCoordPointerEXT)
#define glVertexPointerEXT (*OpenGLVPtr->V_glVertexPointerEXT)
#define gluBuild1DMipmaps (*OpenGLVPtr->V_gluBuild1DMipmaps)
#define gluBuild2DMipmaps (*OpenGLVPtr->V_gluBuild2DMipmaps)
#define gluProject (*OpenGLVPtr->V_gluProject)
#define gluScaleImage (*OpenGLVPtr->V_gluScaleImage)
#define gluUnProject (*OpenGLVPtr->V_gluUnProject)
#define glXCreateGLXPixmap (*OpenGLVPtr->V_glXCreateGLXPixmap)
#define glXDestroyContext (*OpenGLVPtr->V_glXDestroyContext)
#define glXDestroyGLXPixmap (*OpenGLVPtr->V_glXDestroyGLXPixmap)
#define glXGetConfig (*OpenGLVPtr->V_glXGetConfig)
#define glXGetCurrentContext (*OpenGLVPtr->V_glXGetCurrentContext)
#define glXGetCurrentDrawable (*OpenGLVPtr->V_glXGetCurrentDrawable)
#define glXIsDirect (*OpenGLVPtr->V_glXIsDirect)
#define glXMakeCurrent (*OpenGLVPtr->V_glXMakeCurrent)
#define glXQueryExtension (*OpenGLVPtr->V_glXQueryExtension)
#define glXQueryVersion (*OpenGLVPtr->V_glXQueryVersion)
#define glXUseXFont (*OpenGLVPtr->V_glXUseXFont)
#define glXWaitGL (*OpenGLVPtr->V_glXWaitGL)
#define glXWaitX (*OpenGLVPtr->V_glXWaitX)
#endif
#endif
