# Any regular data structure with as many children for each
# branch at each layer.
# Either 2 or 3D.

package PDL::Graphics::TriD::Pyramid;
@ISA=qw/PDL::Graphics::TriD::Object/;

sub new {
	
}

sub togl {
}
