	package PDL::Graphics::OpenGL;
	$Version = 0.5;

	require Exporter;
	require DynaLoader;
	@ISA = qw(Exporter DynaLoader);
	@EXPORT = qw(
		glpOpenWindow
		glXSwapBuffers
		XPending
		glpXNextEvent
		glpXQueryPointer
		glpReadTex
		glpClipPlane
		glpGetClipPlane
		glpLoadMatrixf glpLoadMatrixd glpMultMatrixf glpMultMatrixd
		gluOrtho2D
		gluPerspective
		gluLookAt
		glupPickMatrix

		glpRasterFont
		glpPrintString
		glPolygonOffsetEXT

glAccum
glAlphaFunc
glAreTexturesResidentEXT
glArrayElementEXT
glBegin
glBindTextureEXT
glBitmap
glBlendFunc
glCallList
glCallLists
glClear
glClearAccum
glClearColor
glClearDepth
glClearIndex
glClearStencil
glClipPlane
glColor3b
glColor3bv
glColor3d
glColor3dv
glColor3f
glColor3fv
glColor3i
glColor3iv
glColor3s
glColor3sv
glColor3ub
glColor3ubv
glColor3ui
glColor3uiv
glColor3us
glColor3usv
glColor4b
glColor4bv
glColor4d
glColor4dv
glColor4f
glColor4fv
glColor4i
glColor4iv
glColor4s
glColor4sv
glColor4ub
glColor4ubv
glColor4ui
glColor4uiv
glColor4us
glColor4usv
glColorMask
glColorMaterial
glColorPointerEXT
glCopyPixels
glCullFace
glDeleteLists
glDeleteTexturesEXT
glDepthFunc
glDepthMask
glDepthRange
glDisable
glDrawArraysEXT
glDrawBuffer
glDrawPixels
glEdgeFlag
glEdgeFlagPointerEXT
glEdgeFlagv
glEnable
glEnd
glEndList
glEvalCoord1d
glEvalCoord1dv
glEvalCoord1f
glEvalCoord1fv
glEvalCoord2d
glEvalCoord2dv
glEvalCoord2f
glEvalCoord2fv
glEvalMesh1
glEvalMesh2
glEvalPoint1
glEvalPoint2
glFeedbackBuffer
glFinish
glFlush
glFogf
glFogfv
glFogi
glFogiv
glFrontFace
glFrustum
glGenLists
glGenTexturesEXT
glGetBooleanv
glGetClipPlane
glGetDoublev
glGetError
glGetFloatv
glGetIntegerv
glGetLightfv
glGetLightiv
glGetMapdv
glGetMapfv
glGetMapiv
glGetMaterialfv
glGetMaterialiv
glGetPixelMapfv
glGetPixelMapuiv
glGetPixelMapusv
glGetPolygonStipple
glGetTexEnvfv
glGetTexEnviv
glGetTexGendv
glGetTexGenfv
glGetTexGeniv
glGetTexImage
glGetTexLevelParameterfv
glGetTexLevelParameteriv
glGetTexParameterfv
glGetTexParameteriv
glHint
glIndexMask
glIndexPointerEXT
glIndexd
glIndexdv
glIndexf
glIndexfv
glIndexi
glIndexiv
glIndexs
glIndexsv
glInitNames
glIsEnabled
glIsList
glIsTextureEXT
glLightModelf
glLightModelfv
glLightModeli
glLightModeliv
glLightf
glLightfv
glLighti
glLightiv
glLineStipple
glLineWidth
glListBase
glLoadIdentity
glLoadMatrixd
glLoadMatrixf
glLoadName
glLogicOp
glMap1d
glMap1f
glMap2d
glMap2f
glMapGrid1d
glMapGrid1f
glMapGrid2d
glMapGrid2f
glMaterialf
glMaterialfv
glMateriali
glMaterialiv
glMatrixMode
glMultMatrixd
glMultMatrixf
glNewList
glNormal3b
glNormal3bv
glNormal3d
glNormal3dv
glNormal3f
glNormal3fv
glNormal3i
glNormal3iv
glNormal3s
glNormal3sv
glNormalPointerEXT
glOrtho
glPassThrough
glPixelMapfv
glPixelMapuiv
glPixelMapusv
glPixelStoref
glPixelStorei
glPixelTransferf
glPixelTransferi
glPixelZoom
glPointSize
glPolygonMode
glPolygonStipple
glPopAttrib
glPopMatrix
glPopName
glPrioritizeTexturesEXT
glPushAttrib
glPushMatrix
glPushName
glRasterPos2d
glRasterPos2dv
glRasterPos2f
glRasterPos2fv
glRasterPos2i
glRasterPos2iv
glRasterPos2s
glRasterPos2sv
glRasterPos3d
glRasterPos3dv
glRasterPos3f
glRasterPos3fv
glRasterPos3i
glRasterPos3iv
glRasterPos3s
glRasterPos3sv
glRasterPos4d
glRasterPos4dv
glRasterPos4f
glRasterPos4fv
glRasterPos4i
glRasterPos4iv
glRasterPos4s
glRasterPos4sv
glReadBuffer
glReadPixels
glRectd
glRectdv
glRectf
glRectfv
glRecti
glRectiv
glRects
glRectsv
glRenderMode
glRotated
glRotatef
glScaled
glScalef
glScissor
glSelectBuffer
glShadeModel
glStencilFunc
glStencilMask
glStencilOp
glTexCoord1d
glTexCoord1dv
glTexCoord1f
glTexCoord1fv
glTexCoord1i
glTexCoord1iv
glTexCoord1s
glTexCoord1sv
glTexCoord2d
glTexCoord2dv
glTexCoord2f
glTexCoord2fv
glTexCoord2i
glTexCoord2iv
glTexCoord2s
glTexCoord2sv
glTexCoord3d
glTexCoord3dv
glTexCoord3f
glTexCoord3fv
glTexCoord3i
glTexCoord3iv
glTexCoord3s
glTexCoord3sv
glTexCoord4d
glTexCoord4dv
glTexCoord4f
glTexCoord4fv
glTexCoord4i
glTexCoord4iv
glTexCoord4s
glTexCoord4sv
glTexCoordPointerEXT
glTexEnvf
glTexEnvfv
glTexEnvi
glTexEnviv
glTexGend
glTexGendv
glTexGenf
glTexGenfv
glTexGeni
glTexGeniv
glTexImage1D
glTexImage2D
glTexParameterf
glTexParameterfv
glTexParameteri
glTexParameteriv
glTranslated
glTranslatef
glVertex2d
glVertex2dv
glVertex2f
glVertex2fv
glVertex2i
glVertex2iv
glVertex2s
glVertex2sv
glVertex3d
glVertex3dv
glVertex3f
glVertex3fv
glVertex3i
glVertex3iv
glVertex3s
glVertex3sv
glVertex4d
glVertex4dv
glVertex4f
glVertex4fv
glVertex4i
glVertex4iv
glVertex4s
glVertex4sv
glVertexPointerEXT
glViewport
glXCreateGLXPixmap
glXDestroyContext
glXDestroyGLXPixmap
glXGetConfig
glXGetCurrentContext
glXGetCurrentDrawable
glXIsDirect
glXMakeCurrent
glXQueryExtension
glXQueryVersion
glXUseXFont
glXWaitGL
glXWaitX
gluBuild1DMipmaps
gluBuild2DMipmaps
gluProject
gluScaleImage
gluUnProject

AlreadyGrabbed
AnyModifier
AsyncBoth
AsyncKeyboard
AsyncPointer
Button1
Button1Mask
Button1MotionMask
Button2
Button2Mask
Button2MotionMask
Button3
Button3Mask
Button3MotionMask
Button4
Button4Mask
Button4MotionMask
Button5
Button5Mask
Button5MotionMask
ButtonMotionMask
ButtonPress
ButtonPressMask
ButtonRelease
ButtonReleaseMask
CirculateNotify
CirculateRequest
ClientMessage
ColormapChangeMask
ColormapInstalled
ColormapNotify
ColormapUninstalled
ConfigureNotify
ConfigureRequest
ControlMapIndex
ControlMask
CreateNotify
DestroyNotify
DirectColor
EXT_TEXTURE_OBJECT_MAJOR_REVISION
EXT_TEXTURE_OBJECT_MINOR_REVISION
EXT_TEXTURE_OBJECT_NAME
EXT_VERTEX_ARRAY_MAJOR_REVISION
EXT_VERTEX_ARRAY_MINOR_REVISION
EXT_VERTEX_ARRAY_NAME
EnterNotify
EnterWindowMask
Expose
ExposureMask
FamilyChaos
FamilyDECnet
FamilyInternet
FocusChangeMask
FocusIn
FocusOut
GLX_ACCUM_ALPHA_SIZE
GLX_ACCUM_BLUE_SIZE
GLX_ACCUM_GREEN_SIZE
GLX_ACCUM_RED_SIZE
GLX_ALPHA_SIZE
GLX_AUX_BUFFERS
GLX_BAD_ATTRIBUTE
GLX_BAD_CONTEXT
GLX_BAD_ENUM
GLX_BAD_SCREEN
GLX_BAD_VALUE
GLX_BAD_VISUAL
GLX_BLUE_SIZE
GLX_BUFFER_SIZE
GLX_DEPTH_SIZE
GLX_DIRECT_COLOR_EXT
GLX_DOUBLEBUFFER
GLX_EXTENSIONS
GLX_EXT_import_context
GLX_EXT_visual_info
GLX_EXT_visual_rating
GLX_GRAY_SCALE_EXT
GLX_GREEN_SIZE
GLX_LEVEL
GLX_NONE_EXT
GLX_NON_CONFORMANT_VISUAL_EXT
GLX_NO_EXTENSION
GLX_PSEUDO_COLOR_EXT
GLX_RED_SIZE
GLX_RGBA
GLX_SCREEN_EXT
GLX_SHARE_CONTEXT_EXT
GLX_SLOW_VISUAL_EXT
GLX_STATIC_COLOR_EXT
GLX_STATIC_GRAY_EXT
GLX_STENCIL_SIZE
GLX_STEREO
GLX_TRANSPARENT_ALPHA_VALUE_EXT
GLX_TRANSPARENT_BLUE_VALUE_EXT
GLX_TRANSPARENT_GREEN_VALUE_EXT
GLX_TRANSPARENT_INDEX_EXT
GLX_TRANSPARENT_INDEX_VALUE_EXT
GLX_TRANSPARENT_RED_VALUE_EXT
GLX_TRANSPARENT_RGB_EXT
GLX_TRANSPARENT_TYPE_EXT
GLX_TRUE_COLOR_EXT
GLX_USE_GL
GLX_VENDOR
GLX_VERSION
GLX_VERSION_1_1
GLX_VERSION_1_2
GLX_VISUAL_CAVEAT_EXT
GLX_VISUAL_ID_EXT
GLX_X_VISUAL_TYPE_EXT
GL_2D
GL_2_BYTES
GL_3D
GL_3D_COLOR
GL_3D_COLOR_TEXTURE
GL_3_BYTES
GL_4D_COLOR_TEXTURE
GL_4_BYTES
GL_ABGR_EXT
GL_ACCUM
GL_ACCUM_ALPHA_BITS
GL_ACCUM_BLUE_BITS
GL_ACCUM_BUFFER_BIT
GL_ACCUM_CLEAR_VALUE
GL_ACCUM_GREEN_BITS
GL_ACCUM_RED_BITS
GL_ADD
GL_ALL_ATTRIB_BITS
GL_ALPHA
GL_ALPHA12
GL_ALPHA12_EXT
GL_ALPHA16
GL_ALPHA16_EXT
GL_ALPHA4
GL_ALPHA4_EXT
GL_ALPHA8
GL_ALPHA8_EXT
GL_ALPHA_BIAS
GL_ALPHA_BITS
GL_ALPHA_SCALE
GL_ALPHA_TEST
GL_ALPHA_TEST_FUNC
GL_ALPHA_TEST_REF
GL_ALWAYS
GL_AMBIENT
GL_AMBIENT_AND_DIFFUSE
GL_AND
GL_AND_INVERTED
GL_AND_REVERSE
GL_ATTRIB_STACK_DEPTH
GL_AUTO_NORMAL
GL_AUX0
GL_AUX1
GL_AUX2
GL_AUX3
GL_AUX_BUFFERS
GL_BACK
GL_BACK_LEFT
GL_BACK_RIGHT
GL_BITMAP
GL_BITMAP_TOKEN
GL_BLEND
GL_BLEND_COLOR_EXT
GL_BLEND_DST
GL_BLEND_EQUATION_EXT
GL_BLEND_SRC
GL_BLUE
GL_BLUE_BIAS
GL_BLUE_BITS
GL_BLUE_SCALE
GL_BYTE
GL_C3F_V3F
GL_C4F_N3F_V3F
GL_C4UB_V2F
GL_C4UB_V3F
GL_CCW
GL_CLAMP
GL_CLEAR
GL_CLIENT_ALL_ATTRIB_BITS
GL_CLIENT_ATTRIB_STACK_DEPTH
GL_CLIENT_PIXEL_STORE_BIT
GL_CLIENT_VERTEX_ARRAY_BIT
GL_CLIP_PLANE0
GL_CLIP_PLANE1
GL_CLIP_PLANE2
GL_CLIP_PLANE3
GL_CLIP_PLANE4
GL_CLIP_PLANE5
GL_COEFF
GL_COLOR
GL_COLOR_ARRAY
GL_COLOR_ARRAY_COUNT_EXT
GL_COLOR_ARRAY_EXT
GL_COLOR_ARRAY_POINTER
GL_COLOR_ARRAY_POINTER_EXT
GL_COLOR_ARRAY_SIZE
GL_COLOR_ARRAY_SIZE_EXT
GL_COLOR_ARRAY_STRIDE
GL_COLOR_ARRAY_STRIDE_EXT
GL_COLOR_ARRAY_TYPE
GL_COLOR_ARRAY_TYPE_EXT
GL_COLOR_BUFFER_BIT
GL_COLOR_CLEAR_VALUE
GL_COLOR_INDEX
GL_COLOR_INDEXES
GL_COLOR_LOGIC_OP
GL_COLOR_MATERIAL
GL_COLOR_MATERIAL_FACE
GL_COLOR_MATERIAL_PARAMETER
GL_COLOR_WRITEMASK
GL_COMPILE
GL_COMPILE_AND_EXECUTE
GL_CONSTANT_ALPHA_EXT
GL_CONSTANT_ATTENUATION
GL_CONSTANT_COLOR_EXT
GL_COPY
GL_COPY_INVERTED
GL_COPY_PIXEL_TOKEN
GL_CULL_FACE
GL_CULL_FACE_MODE
GL_CURRENT_BIT
GL_CURRENT_COLOR
GL_CURRENT_INDEX
GL_CURRENT_NORMAL
GL_CURRENT_RASTER_COLOR
GL_CURRENT_RASTER_DISTANCE
GL_CURRENT_RASTER_INDEX
GL_CURRENT_RASTER_POSITION
GL_CURRENT_RASTER_POSITION_VALID
GL_CURRENT_RASTER_TEXTURE_COORDS
GL_CURRENT_TEXTURE_COORDS
GL_CW
GL_DECAL
GL_DECR
GL_DEPTH
GL_DEPTH_BIAS
GL_DEPTH_BITS
GL_DEPTH_BUFFER_BIT
GL_DEPTH_CLEAR_VALUE
GL_DEPTH_COMPONENT
GL_DEPTH_FUNC
GL_DEPTH_RANGE
GL_DEPTH_SCALE
GL_DEPTH_TEST
GL_DEPTH_WRITEMASK
GL_DIFFUSE
GL_DIFFUSE_MATERIAL_COLOR_ES
GL_DITHER
GL_DOMAIN
GL_DONT_CARE
GL_DOUBLE
GL_DOUBLEBUFFER
GL_DOUBLE_EXT
GL_DRAW_BUFFER
GL_DRAW_PIXEL_TOKEN
GL_DST_ALPHA
GL_DST_COLOR
GL_EDGE_FLAG
GL_EDGE_FLAG_ARRAY
GL_EDGE_FLAG_ARRAY_COUNT_EXT
GL_EDGE_FLAG_ARRAY_EXT
GL_EDGE_FLAG_ARRAY_POINTER
GL_EDGE_FLAG_ARRAY_POINTER_EXT
GL_EDGE_FLAG_ARRAY_STRIDE
GL_EDGE_FLAG_ARRAY_STRIDE_EXT
GL_EMISSION
GL_ENABLE_BIT
GL_EQUAL
GL_EQUIV
GL_ES_shaded_texture
GL_EVAL_BIT
GL_EXP
GL_EXP2
GL_EXTENSIONS
GL_EXT_abgr
GL_EXT_blend_color
GL_EXT_blend_minmax
GL_EXT_blend_subtract
GL_EXT_texture_object
GL_EXT_vertex_array
GL_EYE_LINEAR
GL_EYE_PLANE
GL_FALSE
GL_FASTEST
GL_FEEDBACK
GL_FEEDBACK_BUFFER_POINTER
GL_FEEDBACK_BUFFER_SIZE
GL_FEEDBACK_BUFFER_TYPE
GL_FILL
GL_FINAL_COLOR_ES
GL_FLAT
GL_FLOAT
GL_FOG
GL_FOG_BIT
GL_FOG_COLOR
GL_FOG_DENSITY
GL_FOG_END
GL_FOG_HINT
GL_FOG_INDEX
GL_FOG_MODE
GL_FOG_START
GL_FRONT
GL_FRONT_AND_BACK
GL_FRONT_FACE
GL_FRONT_LEFT
GL_FRONT_RIGHT
GL_FUNC_ADD_EXT
GL_FUNC_REVERSE_SUBTRACT_EXT
GL_FUNC_SUBTRACT_EXT
GL_GEQUAL
GL_GREATER
GL_GREEN
GL_GREEN_BIAS
GL_GREEN_BITS
GL_GREEN_SCALE
GL_HINT_BIT
GL_INCR
GL_INDEX_ARRAY
GL_INDEX_ARRAY_COUNT_EXT
GL_INDEX_ARRAY_EXT
GL_INDEX_ARRAY_POINTER
GL_INDEX_ARRAY_POINTER_EXT
GL_INDEX_ARRAY_STRIDE
GL_INDEX_ARRAY_STRIDE_EXT
GL_INDEX_ARRAY_TYPE
GL_INDEX_ARRAY_TYPE_EXT
GL_INDEX_BITS
GL_INDEX_CLEAR_VALUE
GL_INDEX_LOGIC_OP
GL_INDEX_MODE
GL_INDEX_OFFSET
GL_INDEX_SHIFT
GL_INDEX_WRITEMASK
GL_INT
GL_INTENSITY
GL_INTENSITY12
GL_INTENSITY12_EXT
GL_INTENSITY16
GL_INTENSITY16_EXT
GL_INTENSITY4
GL_INTENSITY4_EXT
GL_INTENSITY8
GL_INTENSITY8_EXT
GL_INTENSITY_EXT
GL_INVALID_ENUM
GL_INVALID_OPERATION
GL_INVALID_VALUE
GL_INVERT
GL_KEEP
GL_LEFT
GL_LEQUAL
GL_LESS
GL_LIGHT0
GL_LIGHT1
GL_LIGHT2
GL_LIGHT3
GL_LIGHT4
GL_LIGHT5
GL_LIGHT6
GL_LIGHT7
GL_LIGHTING
GL_LIGHTING_BIT
GL_LIGHT_MODEL_AMBIENT
GL_LIGHT_MODEL_LOCAL_VIEWER
GL_LIGHT_MODEL_TWO_SIDE
GL_LINE
GL_LINEAR
GL_LINEAR_ATTENUATION
GL_LINEAR_MIPMAP_LINEAR
GL_LINEAR_MIPMAP_NEAREST
GL_LINES
GL_LINE_BIT
GL_LINE_LOOP
GL_LINE_RESET_TOKEN
GL_LINE_SMOOTH
GL_LINE_SMOOTH_HINT
GL_LINE_STIPPLE
GL_LINE_STIPPLE_PATTERN
GL_LINE_STIPPLE_REPEAT
GL_LINE_STRIP
GL_LINE_TOKEN
GL_LINE_WIDTH
GL_LINE_WIDTH_GRANULARITY
GL_LINE_WIDTH_RANGE
GL_LIST_BASE
GL_LIST_BIT
GL_LIST_INDEX
GL_LIST_MODE
GL_LOAD
GL_LOGIC_OP
GL_LOGIC_OP_MODE
GL_LUMINANCE
GL_LUMINANCE12
GL_LUMINANCE12_ALPHA12
GL_LUMINANCE12_ALPHA12_EXT
GL_LUMINANCE12_ALPHA4
GL_LUMINANCE12_ALPHA4_EXT
GL_LUMINANCE12_EXT
GL_LUMINANCE16
GL_LUMINANCE16_ALPHA16
GL_LUMINANCE16_ALPHA16_EXT
GL_LUMINANCE16_EXT
GL_LUMINANCE4
GL_LUMINANCE4_ALPHA4
GL_LUMINANCE4_ALPHA4_EXT
GL_LUMINANCE4_EXT
GL_LUMINANCE6_ALPHA2
GL_LUMINANCE6_ALPHA2_EXT
GL_LUMINANCE8
GL_LUMINANCE8_ALPHA8
GL_LUMINANCE8_ALPHA8_EXT
GL_LUMINANCE8_EXT
GL_LUMINANCE_ALPHA
GL_MAP1_COLOR_4
GL_MAP1_GRID_DOMAIN
GL_MAP1_GRID_SEGMENTS
GL_MAP1_INDEX
GL_MAP1_NORMAL
GL_MAP1_TEXTURE_COORD_1
GL_MAP1_TEXTURE_COORD_2
GL_MAP1_TEXTURE_COORD_3
GL_MAP1_TEXTURE_COORD_4
GL_MAP1_VERTEX_3
GL_MAP1_VERTEX_4
GL_MAP2_COLOR_4
GL_MAP2_GRID_DOMAIN
GL_MAP2_GRID_SEGMENTS
GL_MAP2_INDEX
GL_MAP2_NORMAL
GL_MAP2_TEXTURE_COORD_1
GL_MAP2_TEXTURE_COORD_2
GL_MAP2_TEXTURE_COORD_3
GL_MAP2_TEXTURE_COORD_4
GL_MAP2_VERTEX_3
GL_MAP2_VERTEX_4
GL_MAP_COLOR
GL_MAP_STENCIL
GL_MATRIX_MODE
GL_MAX_ATTRIB_STACK_DEPTH
GL_MAX_CLIENT_ATTRIB_STACK_DEPTH
GL_MAX_CLIP_PLANES
GL_MAX_EVAL_ORDER
GL_MAX_EXT
GL_MAX_LIGHTS
GL_MAX_LIST_NESTING
GL_MAX_MODELVIEW_STACK_DEPTH
GL_MAX_NAME_STACK_DEPTH
GL_MAX_PIXEL_MAP_TABLE
GL_MAX_PROJECTION_STACK_DEPTH
GL_MAX_TEXTURE_SIZE
GL_MAX_TEXTURE_STACK_DEPTH
GL_MAX_VIEWPORT_DIMS
GL_MIN_EXT
GL_MODELVIEW
GL_MODELVIEW_MATRIX
GL_MODELVIEW_STACK_DEPTH
GL_MODULATE
GL_MULT
GL_N3F_V3F
GL_NAME_STACK_DEPTH
GL_NAND
GL_NEAREST
GL_NEAREST_MIPMAP_LINEAR
GL_NEAREST_MIPMAP_NEAREST
GL_NEVER
GL_NICEST
GL_NONE
GL_NOOP
GL_NOR
GL_NORMALIZE
GL_NORMAL_ARRAY
GL_NORMAL_ARRAY_COUNT_EXT
GL_NORMAL_ARRAY_EXT
GL_NORMAL_ARRAY_POINTER
GL_NORMAL_ARRAY_POINTER_EXT
GL_NORMAL_ARRAY_STRIDE
GL_NORMAL_ARRAY_STRIDE_EXT
GL_NORMAL_ARRAY_TYPE
GL_NORMAL_ARRAY_TYPE_EXT
GL_NOTEQUAL
GL_NO_ERROR
GL_OBJECT_LINEAR
GL_OBJECT_PLANE
GL_ONE
GL_ONE_MINUS_CONSTANT_ALPHA_EXT
GL_ONE_MINUS_CONSTANT_COLOR_EXT
GL_ONE_MINUS_DST_ALPHA
GL_ONE_MINUS_DST_COLOR
GL_ONE_MINUS_SRC_ALPHA
GL_ONE_MINUS_SRC_COLOR
GL_OR
GL_ORDER
GL_OR_INVERTED
GL_OR_REVERSE
GL_OUT_OF_MEMORY
GL_PACK_ALIGNMENT
GL_PACK_LSB_FIRST
GL_PACK_ROW_LENGTH
GL_PACK_SKIP_PIXELS
GL_PACK_SKIP_ROWS
GL_PACK_SWAP_BYTES
GL_PASS_THROUGH_TOKEN
GL_PERSPECTIVE_CORRECTION_HINT
GL_PIXEL_MAP_A_TO_A
GL_PIXEL_MAP_A_TO_A_SIZE
GL_PIXEL_MAP_B_TO_B
GL_PIXEL_MAP_B_TO_B_SIZE
GL_PIXEL_MAP_G_TO_G
GL_PIXEL_MAP_G_TO_G_SIZE
GL_PIXEL_MAP_I_TO_A
GL_PIXEL_MAP_I_TO_A_SIZE
GL_PIXEL_MAP_I_TO_B
GL_PIXEL_MAP_I_TO_B_SIZE
GL_PIXEL_MAP_I_TO_G
GL_PIXEL_MAP_I_TO_G_SIZE
GL_PIXEL_MAP_I_TO_I
GL_PIXEL_MAP_I_TO_I_SIZE
GL_PIXEL_MAP_I_TO_R
GL_PIXEL_MAP_I_TO_R_SIZE
GL_PIXEL_MAP_R_TO_R
GL_PIXEL_MAP_R_TO_R_SIZE
GL_PIXEL_MAP_S_TO_S
GL_PIXEL_MAP_S_TO_S_SIZE
GL_PIXEL_MODE_BIT
GL_POINT
GL_POINTS
GL_POINT_BIT
GL_POINT_SIZE
GL_POINT_SIZE_GRANULARITY
GL_POINT_SIZE_RANGE
GL_POINT_SMOOTH
GL_POINT_SMOOTH_HINT
GL_POINT_TOKEN
GL_POLYGON
GL_POLYGON_BIT
GL_POLYGON_MODE
GL_POLYGON_OFFSET_BIAS_EXT
GL_POLYGON_OFFSET_EXT
GL_POLYGON_OFFSET_FACTOR
GL_POLYGON_OFFSET_FACTOR_EXT
GL_POLYGON_OFFSET_FILL
GL_POLYGON_OFFSET_LINE
GL_POLYGON_OFFSET_POINT
GL_POLYGON_OFFSET_UNITS
GL_POLYGON_SMOOTH
GL_POLYGON_SMOOTH_HINT
GL_POLYGON_STIPPLE
GL_POLYGON_STIPPLE_BIT
GL_POLYGON_TOKEN
GL_POSITION
GL_PROJECTION
GL_PROJECTION_MATRIX
GL_PROJECTION_STACK_DEPTH
GL_PROXY_TEXTURE_1D
GL_PROXY_TEXTURE_1D_EXT
GL_PROXY_TEXTURE_2D
GL_PROXY_TEXTURE_2D_EXT
GL_Q
GL_QUADRATIC_ATTENUATION
GL_QUADS
GL_QUAD_STRIP
GL_R
GL_R3_G3_B2
GL_READ_BUFFER
GL_RED
GL_RED_BIAS
GL_RED_BITS
GL_RED_SCALE
GL_RENDER
GL_RENDERER
GL_RENDER_MODE
GL_REPEAT
GL_REPLACE
GL_REPLACE_EXT
GL_RETURN
GL_RGB
GL_RGB10
GL_RGB10_A2
GL_RGB10_A2_EXT
GL_RGB10_EXT
GL_RGB12
GL_RGB12_EXT
GL_RGB16
GL_RGB16_EXT
GL_RGB2_EXT
GL_RGB4
GL_RGB4_EXT
GL_RGB5
GL_RGB5_A1
GL_RGB5_A1_EXT
GL_RGB5_EXT
GL_RGB8
GL_RGB8_EXT
GL_RGBA
GL_RGBA12
GL_RGBA12_EXT
GL_RGBA16
GL_RGBA16_EXT
GL_RGBA2
GL_RGBA2_EXT
GL_RGBA4
GL_RGBA4_EXT
GL_RGBA8
GL_RGBA8_EXT
GL_RGBA_MODE
GL_RIGHT
GL_S
GL_SCISSOR_BIT
GL_SCISSOR_BOX
GL_SCISSOR_TEST
GL_SELECT
GL_SELECTION_BUFFER_POINTER
GL_SELECTION_BUFFER_SIZE
GL_SET
GL_SHADE_MODEL
GL_SHININESS
GL_SHORT
GL_SMOOTH
GL_SPECULAR
GL_SPHERE_MAP
GL_SPOT_CUTOFF
GL_SPOT_DIRECTION
GL_SPOT_EXPONENT
GL_SRC_ALPHA
GL_SRC_ALPHA_SATURATE
GL_SRC_COLOR
GL_STACK_OVERFLOW
GL_STACK_UNDERFLOW
GL_STENCIL
GL_STENCIL_BITS
GL_STENCIL_BUFFER_BIT
GL_STENCIL_CLEAR_VALUE
GL_STENCIL_FAIL
GL_STENCIL_FUNC
GL_STENCIL_INDEX
GL_STENCIL_PASS_DEPTH_FAIL
GL_STENCIL_PASS_DEPTH_PASS
GL_STENCIL_REF
GL_STENCIL_TEST
GL_STENCIL_VALUE_MASK
GL_STENCIL_WRITEMASK
GL_STEREO
GL_SUBPIXEL_BITS
GL_T
GL_T2F_C3F_V3F
GL_T2F_C4F_N3F_V3F
GL_T2F_C4UB_V3F
GL_T2F_N3F_V3F
GL_T2F_V3F
GL_T4F_C4F_N3F_V4F
GL_T4F_V4F
GL_TEXTURE
GL_TEXTURE_1D
GL_TEXTURE_1D_BINDING_EXT
GL_TEXTURE_2D
GL_TEXTURE_2D_BINDING_EXT
GL_TEXTURE_3D_BINDING_EXT
GL_TEXTURE_ALPHA_SIZE
GL_TEXTURE_ALPHA_SIZE_EXT
GL_TEXTURE_BINDING_1D
GL_TEXTURE_BINDING_2D
GL_TEXTURE_BIT
GL_TEXTURE_BLUE_SIZE
GL_TEXTURE_BLUE_SIZE_EXT
GL_TEXTURE_BORDER
GL_TEXTURE_BORDER_COLOR
GL_TEXTURE_COMPONENTS
GL_TEXTURE_COORD_ARRAY
GL_TEXTURE_COORD_ARRAY_COUNT_EXT
GL_TEXTURE_COORD_ARRAY_EXT
GL_TEXTURE_COORD_ARRAY_POINTER
GL_TEXTURE_COORD_ARRAY_POINTER_EXT
GL_TEXTURE_COORD_ARRAY_SIZE
GL_TEXTURE_COORD_ARRAY_SIZE_EXT
GL_TEXTURE_COORD_ARRAY_STRIDE
GL_TEXTURE_COORD_ARRAY_STRIDE_EXT
GL_TEXTURE_COORD_ARRAY_TYPE
GL_TEXTURE_COORD_ARRAY_TYPE_EXT
GL_TEXTURE_ENV
GL_TEXTURE_ENV_COLOR
GL_TEXTURE_ENV_COMPONENT_ES
GL_TEXTURE_ENV_MODE
GL_TEXTURE_GEN_MODE
GL_TEXTURE_GEN_Q
GL_TEXTURE_GEN_R
GL_TEXTURE_GEN_S
GL_TEXTURE_GEN_T
GL_TEXTURE_GREEN_SIZE
GL_TEXTURE_GREEN_SIZE_EXT
GL_TEXTURE_HEIGHT
GL_TEXTURE_INTENSITY_SIZE
GL_TEXTURE_INTENSITY_SIZE_EXT
GL_TEXTURE_INTERNAL_FORMAT
GL_TEXTURE_LUMINANCE_SIZE
GL_TEXTURE_LUMINANCE_SIZE_EXT
GL_TEXTURE_MAG_FILTER
GL_TEXTURE_MATRIX
GL_TEXTURE_MIN_FILTER
GL_TEXTURE_PRIORITY
GL_TEXTURE_PRIORITY_EXT
GL_TEXTURE_RED_SIZE
GL_TEXTURE_RED_SIZE_EXT
GL_TEXTURE_RESIDENT
GL_TEXTURE_RESIDENT_EXT
GL_TEXTURE_STACK_DEPTH
GL_TEXTURE_TOO_LARGE_EXT
GL_TEXTURE_WIDTH
GL_TEXTURE_WRAP_S
GL_TEXTURE_WRAP_T
GL_TRANSFORM_BIT
GL_TRIANGLES
GL_TRIANGLE_FAN
GL_TRIANGLE_STRIP
GL_TRUE
GL_UNPACK_ALIGNMENT
GL_UNPACK_LSB_FIRST
GL_UNPACK_ROW_LENGTH
GL_UNPACK_SKIP_PIXELS
GL_UNPACK_SKIP_ROWS
GL_UNPACK_SWAP_BYTES
GL_UNSIGNED_BYTE
GL_UNSIGNED_INT
GL_UNSIGNED_SHORT
GL_V2F
GL_V3F
GL_VENDOR
GL_VERSION
GL_VERSION_1_1
GL_VERTEX_ARRAY
GL_VERTEX_ARRAY_COUNT_EXT
GL_VERTEX_ARRAY_EXT
GL_VERTEX_ARRAY_POINTER
GL_VERTEX_ARRAY_POINTER_EXT
GL_VERTEX_ARRAY_SIZE
GL_VERTEX_ARRAY_SIZE_EXT
GL_VERTEX_ARRAY_STRIDE
GL_VERTEX_ARRAY_STRIDE_EXT
GL_VERTEX_ARRAY_TYPE
GL_VERTEX_ARRAY_TYPE_EXT
GL_VIEWPORT
GL_VIEWPORT_BIT
GL_XOR
GL_ZERO
GL_ZOOM_X
GL_ZOOM_Y
GrabFrozen
GrabInvalidTime
GrabModeAsync
GrabModeSync
GrabNotViewable
GrabSuccess
GraphicsExpose
GravityNotify
GrayScale
KeyPress
KeyPressMask
KeyRelease
KeyReleaseMask
KeymapNotify
KeymapStateMask
LASTEvent
LeaveNotify
LeaveWindowMask
LockMapIndex
LockMask
MapNotify
MapRequest
MappingNotify
Mod1MapIndex
Mod1Mask
Mod2MapIndex
Mod2Mask
Mod3MapIndex
Mod3Mask
Mod4MapIndex
Mod4Mask
Mod5MapIndex
Mod5Mask
MotionNotify
NoEventMask
NoExpose
NotifyAncestor
NotifyDetailNone
NotifyGrab
NotifyHint
NotifyInferior
NotifyNonlinear
NotifyNonlinearVirtual
NotifyNormal
NotifyPointer
NotifyPointerRoot
NotifyUngrab
NotifyVirtual
NotifyWhileGrabbed
OwnerGrabButtonMask
PlaceOnBottom
PlaceOnTop
PointerMotionHintMask
PointerMotionMask
PropertyChangeMask
PropertyDelete
PropertyNewValue
PropertyNotify
PseudoColor
ReparentNotify
ReplayKeyboard
ReplayPointer
ResizeRedirectMask
ResizeRequest
SelectionClear
SelectionNotify
SelectionRequest
ShiftMapIndex
ShiftMask
StaticColor
StaticGray
StructureNotifyMask
SubstructureNotifyMask
SubstructureRedirectMask
SyncBoth
SyncKeyboard
SyncPointer
TrueColor
UnmapNotify
VisibilityChangeMask
VisibilityFullyObscured
VisibilityNotify
VisibilityPartiallyObscured
VisibilityUnobscured
X_PROTOCOL
X_PROTOCOL_REVISION
	);
	bootstrap PDL::Graphics::OpenGL;
sub AlreadyGrabbed () {1}
sub AnyModifier () {(1<<15)}
sub AsyncBoth () {6}
sub AsyncKeyboard () {3}
sub AsyncPointer () {0}
sub Button1 () {1}
sub Button1Mask () {(1<<8)}
sub Button1MotionMask () {(1<<8)}
sub Button2 () {2}
sub Button2Mask () {(1<<9)}
sub Button2MotionMask () {(1<<9)}
sub Button3 () {3}
sub Button3Mask () {(1<<10)}
sub Button3MotionMask () {(1<<10)}
sub Button4 () {4}
sub Button4Mask () {(1<<11)}
sub Button4MotionMask () {(1<<11)}
sub Button5 () {5}
sub Button5Mask () {(1<<12)}
sub Button5MotionMask () {(1<<12)}
sub ButtonMotionMask () {(1<<13)}
sub ButtonPress () {4}
sub ButtonPressMask () {(1<<2)}
sub ButtonRelease () {5}
sub ButtonReleaseMask () {(1<<3)}
sub CirculateNotify () {26}
sub CirculateRequest () {27}
sub ClientMessage () {33}
sub ColormapChangeMask () {(1<<23)}
sub ColormapInstalled () {1}
sub ColormapNotify () {32}
sub ColormapUninstalled () {0}
sub ConfigureNotify () {22}
sub ConfigureRequest () {23}
sub ControlMapIndex () {2}
sub ControlMask () {(1<<2)}
sub CreateNotify () {16}
sub DestroyNotify () {17}
sub DirectColor () {5}
sub EXT_TEXTURE_OBJECT_MAJOR_REVISION () {1}
sub EXT_TEXTURE_OBJECT_MINOR_REVISION () {0}
sub EXT_TEXTURE_OBJECT_NAME () {"GL_EXT_texture_object"}
sub EXT_VERTEX_ARRAY_MAJOR_REVISION () {1}
sub EXT_VERTEX_ARRAY_MINOR_REVISION () {0}
sub EXT_VERTEX_ARRAY_NAME () {"GL_EXT_vertex_array"}
sub EnterNotify () {7}
sub EnterWindowMask () {(1<<4)}
sub Expose () {12}
sub ExposureMask () {(1<<15)}
sub FamilyChaos () {2}
sub FamilyDECnet () {1}
sub FamilyInternet () {0}
sub FocusChangeMask () {(1<<21)}
sub FocusIn () {9}
sub FocusOut () {10}
sub GLX_ACCUM_ALPHA_SIZE () {17}
sub GLX_ACCUM_BLUE_SIZE () {16}
sub GLX_ACCUM_GREEN_SIZE () {15}
sub GLX_ACCUM_RED_SIZE () {14}
sub GLX_ALPHA_SIZE () {11}
sub GLX_AUX_BUFFERS () {7}
sub GLX_BAD_ATTRIBUTE () {2}
sub GLX_BAD_CONTEXT () {5}
sub GLX_BAD_ENUM () {7}
sub GLX_BAD_SCREEN () {1}
sub GLX_BAD_VALUE () {6}
sub GLX_BAD_VISUAL () {4}
sub GLX_BLUE_SIZE () {10}
sub GLX_BUFFER_SIZE () {2}
sub GLX_DEPTH_SIZE () {12}
sub GLX_DIRECT_COLOR_EXT () {0x8003}
sub GLX_DOUBLEBUFFER () {5}
sub GLX_EXTENSIONS () {0x1}
sub GLX_EXT_import_context () {1}
sub GLX_EXT_visual_info () {1}
sub GLX_EXT_visual_rating () {1}
sub GLX_GRAY_SCALE_EXT () {0x8006}
sub GLX_GREEN_SIZE () {9}
sub GLX_LEVEL () {3}
sub GLX_NONE_EXT () {0x8000}
sub GLX_NON_CONFORMANT_VISUAL_EXT () {0x800D}
sub GLX_NO_EXTENSION () {3}
sub GLX_PSEUDO_COLOR_EXT () {0x8004}
sub GLX_RED_SIZE () {8}
sub GLX_RGBA () {4}
sub GLX_SCREEN_EXT () {0x800C}
sub GLX_SHARE_CONTEXT_EXT () {0x800A}
sub GLX_SLOW_VISUAL_EXT () {0x8001}
sub GLX_STATIC_COLOR_EXT () {0x8005}
sub GLX_STATIC_GRAY_EXT () {0x8007}
sub GLX_STENCIL_SIZE () {13}
sub GLX_STEREO () {6}
sub GLX_TRANSPARENT_ALPHA_VALUE_EXT () {0x28}
sub GLX_TRANSPARENT_BLUE_VALUE_EXT () {0x27}
sub GLX_TRANSPARENT_GREEN_VALUE_EXT () {0x26}
sub GLX_TRANSPARENT_INDEX_EXT () {0x8009}
sub GLX_TRANSPARENT_INDEX_VALUE_EXT () {0x24}
sub GLX_TRANSPARENT_RED_VALUE_EXT () {0x25}
sub GLX_TRANSPARENT_RGB_EXT () {0x8008}
sub GLX_TRANSPARENT_TYPE_EXT () {0x23}
sub GLX_TRUE_COLOR_EXT () {0x8002}
sub GLX_USE_GL () {1}
sub GLX_VENDOR () {0x2}
sub GLX_VERSION () {0x3}
sub GLX_VERSION_1_1 () {1}
sub GLX_VERSION_1_2 () {1}
sub GLX_VISUAL_CAVEAT_EXT () {0x20}
sub GLX_VISUAL_ID_EXT () {0x800B}
sub GLX_X_VISUAL_TYPE_EXT () {0x22}
sub GL_2D () {0x0600}
sub GL_2_BYTES () {0x1407}
sub GL_3D () {0x0601}
sub GL_3D_COLOR () {0x0602}
sub GL_3D_COLOR_TEXTURE () {0x0603}
sub GL_3_BYTES () {0x1408}
sub GL_4D_COLOR_TEXTURE () {0x0604}
sub GL_4_BYTES () {0x1409}
sub GL_ABGR_EXT () {0x8000}
sub GL_ACCUM () {0x0100}
sub GL_ACCUM_ALPHA_BITS () {0x0D5B}
sub GL_ACCUM_BLUE_BITS () {0x0D5A}
sub GL_ACCUM_BUFFER_BIT () {0x00000200}
sub GL_ACCUM_CLEAR_VALUE () {0x0B80}
sub GL_ACCUM_GREEN_BITS () {0x0D59}
sub GL_ACCUM_RED_BITS () {0x0D58}
sub GL_ADD () {0x0104}
sub GL_ALL_ATTRIB_BITS () {0x000fffff}
sub GL_ALPHA () {0x1906}
sub GL_ALPHA12 () {0x803D}
sub GL_ALPHA12_EXT () {0x803D}
sub GL_ALPHA16 () {0x803E}
sub GL_ALPHA16_EXT () {0x803E}
sub GL_ALPHA4 () {0x803B}
sub GL_ALPHA4_EXT () {0x803B}
sub GL_ALPHA8 () {0x803C}
sub GL_ALPHA8_EXT () {0x803C}
sub GL_ALPHA_BIAS () {0x0D1D}
sub GL_ALPHA_BITS () {0x0D55}
sub GL_ALPHA_SCALE () {0x0D1C}
sub GL_ALPHA_TEST () {0x0BC0}
sub GL_ALPHA_TEST_FUNC () {0x0BC1}
sub GL_ALPHA_TEST_REF () {0x0BC2}
sub GL_ALWAYS () {0x0207}
sub GL_AMBIENT () {0x1200}
sub GL_AMBIENT_AND_DIFFUSE () {0x1602}
sub GL_AND () {0x1501}
sub GL_AND_INVERTED () {0x1504}
sub GL_AND_REVERSE () {0x1502}
sub GL_ATTRIB_STACK_DEPTH () {0x0BB0}
sub GL_AUTO_NORMAL () {0x0D80}
sub GL_AUX0 () {0x0409}
sub GL_AUX1 () {0x040A}
sub GL_AUX2 () {0x040B}
sub GL_AUX3 () {0x040C}
sub GL_AUX_BUFFERS () {0x0C00}
sub GL_BACK () {0x0405}
sub GL_BACK_LEFT () {0x0402}
sub GL_BACK_RIGHT () {0x0403}
sub GL_BITMAP () {0x1A00}
sub GL_BITMAP_TOKEN () {0x0704}
sub GL_BLEND () {0x0BE2}
sub GL_BLEND_COLOR_EXT () {0x8005}
sub GL_BLEND_DST () {0x0BE0}
sub GL_BLEND_EQUATION_EXT () {0x8009}
sub GL_BLEND_SRC () {0x0BE1}
sub GL_BLUE () {0x1905}
sub GL_BLUE_BIAS () {0x0D1B}
sub GL_BLUE_BITS () {0x0D54}
sub GL_BLUE_SCALE () {0x0D1A}
sub GL_BYTE () {0x1400}
sub GL_C3F_V3F () {0x2A24}
sub GL_C4F_N3F_V3F () {0x2A26}
sub GL_C4UB_V2F () {0x2A22}
sub GL_C4UB_V3F () {0x2A23}
sub GL_CCW () {0x0901}
sub GL_CLAMP () {0x2900}
sub GL_CLEAR () {0x1500}
sub GL_CLIENT_ALL_ATTRIB_BITS () {0xffffffff}
sub GL_CLIENT_ATTRIB_STACK_DEPTH () {0x0BB1}
sub GL_CLIENT_PIXEL_STORE_BIT () {0x00000001}
sub GL_CLIENT_VERTEX_ARRAY_BIT () {0x00000002}
sub GL_CLIP_PLANE0 () {0x3000}
sub GL_CLIP_PLANE1 () {0x3001}
sub GL_CLIP_PLANE2 () {0x3002}
sub GL_CLIP_PLANE3 () {0x3003}
sub GL_CLIP_PLANE4 () {0x3004}
sub GL_CLIP_PLANE5 () {0x3005}
sub GL_COEFF () {0x0A00}
sub GL_COLOR () {0x1800}
sub GL_COLOR_ARRAY () {0x8076}
sub GL_COLOR_ARRAY_COUNT_EXT () {0x8084}
sub GL_COLOR_ARRAY_EXT () {0x8076}
sub GL_COLOR_ARRAY_POINTER () {0x8090}
sub GL_COLOR_ARRAY_POINTER_EXT () {0x8090}
sub GL_COLOR_ARRAY_SIZE () {0x8081}
sub GL_COLOR_ARRAY_SIZE_EXT () {0x8081}
sub GL_COLOR_ARRAY_STRIDE () {0x8083}
sub GL_COLOR_ARRAY_STRIDE_EXT () {0x8083}
sub GL_COLOR_ARRAY_TYPE () {0x8082}
sub GL_COLOR_ARRAY_TYPE_EXT () {0x8082}
sub GL_COLOR_BUFFER_BIT () {0x00004000}
sub GL_COLOR_CLEAR_VALUE () {0x0C22}
sub GL_COLOR_INDEX () {0x1900}
sub GL_COLOR_INDEXES () {0x1603}
sub GL_COLOR_LOGIC_OP () {0x0BF2}
sub GL_COLOR_MATERIAL () {0x0B57}
sub GL_COLOR_MATERIAL_FACE () {0x0B55}
sub GL_COLOR_MATERIAL_PARAMETER () {0x0B56}
sub GL_COLOR_WRITEMASK () {0x0C23}
sub GL_COMPILE () {0x1300}
sub GL_COMPILE_AND_EXECUTE () {0x1301}
sub GL_CONSTANT_ALPHA_EXT () {0x8003}
sub GL_CONSTANT_ATTENUATION () {0x1207}
sub GL_CONSTANT_COLOR_EXT () {0x8001}
sub GL_COPY () {0x1503}
sub GL_COPY_INVERTED () {0x150C}
sub GL_COPY_PIXEL_TOKEN () {0x0706}
sub GL_CULL_FACE () {0x0B44}
sub GL_CULL_FACE_MODE () {0x0B45}
sub GL_CURRENT_BIT () {0x00000001}
sub GL_CURRENT_COLOR () {0x0B00}
sub GL_CURRENT_INDEX () {0x0B01}
sub GL_CURRENT_NORMAL () {0x0B02}
sub GL_CURRENT_RASTER_COLOR () {0x0B04}
sub GL_CURRENT_RASTER_DISTANCE () {0x0B09}
sub GL_CURRENT_RASTER_INDEX () {0x0B05}
sub GL_CURRENT_RASTER_POSITION () {0x0B07}
sub GL_CURRENT_RASTER_POSITION_VALID () {0x0B08}
sub GL_CURRENT_RASTER_TEXTURE_COORDS () {0x0B06}
sub GL_CURRENT_TEXTURE_COORDS () {0x0B03}
sub GL_CW () {0x0900}
sub GL_DECAL () {0x2101}
sub GL_DECR () {0x1E03}
sub GL_DEPTH () {0x1801}
sub GL_DEPTH_BIAS () {0x0D1F}
sub GL_DEPTH_BITS () {0x0D56}
sub GL_DEPTH_BUFFER_BIT () {0x00000100}
sub GL_DEPTH_CLEAR_VALUE () {0x0B73}
sub GL_DEPTH_COMPONENT () {0x1902}
sub GL_DEPTH_FUNC () {0x0B74}
sub GL_DEPTH_RANGE () {0x0B70}
sub GL_DEPTH_SCALE () {0x0D1E}
sub GL_DEPTH_TEST () {0x0B71}
sub GL_DEPTH_WRITEMASK () {0x0B72}
sub GL_DIFFUSE () {0x1201}
sub GL_DIFFUSE_MATERIAL_COLOR_ES () {108002}
sub GL_DITHER () {0x0BD0}
sub GL_DOMAIN () {0x0A02}
sub GL_DONT_CARE () {0x1100}
sub GL_DOUBLE () {0x140A}
sub GL_DOUBLEBUFFER () {0x0C32}
sub GL_DOUBLE_EXT () {GL_DOUBLE}
sub GL_DRAW_BUFFER () {0x0C01}
sub GL_DRAW_PIXEL_TOKEN () {0x0705}
sub GL_DST_ALPHA () {0x0304}
sub GL_DST_COLOR () {0x0306}
sub GL_EDGE_FLAG () {0x0B43}
sub GL_EDGE_FLAG_ARRAY () {0x8079}
sub GL_EDGE_FLAG_ARRAY_COUNT_EXT () {0x808D}
sub GL_EDGE_FLAG_ARRAY_EXT () {0x8079}
sub GL_EDGE_FLAG_ARRAY_POINTER () {0x8093}
sub GL_EDGE_FLAG_ARRAY_POINTER_EXT () {0x8093}
sub GL_EDGE_FLAG_ARRAY_STRIDE () {0x808C}
sub GL_EDGE_FLAG_ARRAY_STRIDE_EXT () {0x808C}
sub GL_EMISSION () {0x1600}
sub GL_ENABLE_BIT () {0x00002000}
sub GL_EQUAL () {0x0202}
sub GL_EQUIV () {0x1509}
sub GL_ES_shaded_texture () {1}
sub GL_EVAL_BIT () {0x00010000}
sub GL_EXP () {0x0800}
sub GL_EXP2 () {0x0801}
sub GL_EXTENSIONS () {0x1F03}
sub GL_EXT_abgr () {1}
sub GL_EXT_blend_color () {1}
sub GL_EXT_blend_minmax () {1}
sub GL_EXT_blend_subtract () {1}
sub GL_EXT_texture_object () {1}
sub GL_EXT_vertex_array () {1}
sub GL_EYE_LINEAR () {0x2400}
sub GL_EYE_PLANE () {0x2502}
sub GL_FALSE () {0}
sub GL_FASTEST () {0x1101}
sub GL_FEEDBACK () {0x1C01}
sub GL_FEEDBACK_BUFFER_POINTER () {0x0DF0}
sub GL_FEEDBACK_BUFFER_SIZE () {0x0DF1}
sub GL_FEEDBACK_BUFFER_TYPE () {0x0DF2}
sub GL_FILL () {0x1B02}
sub GL_FINAL_COLOR_ES () {108001}
sub GL_FLAT () {0x1D00}
sub GL_FLOAT () {0x1406}
sub GL_FOG () {0x0B60}
sub GL_FOG_BIT () {0x00000080}
sub GL_FOG_COLOR () {0x0B66}
sub GL_FOG_DENSITY () {0x0B62}
sub GL_FOG_END () {0x0B64}
sub GL_FOG_HINT () {0x0C54}
sub GL_FOG_INDEX () {0x0B61}
sub GL_FOG_MODE () {0x0B65}
sub GL_FOG_START () {0x0B63}
sub GL_FRONT () {0x0404}
sub GL_FRONT_AND_BACK () {0x0408}
sub GL_FRONT_FACE () {0x0B46}
sub GL_FRONT_LEFT () {0x0400}
sub GL_FRONT_RIGHT () {0x0401}
sub GL_FUNC_ADD_EXT () {0x8006}
sub GL_FUNC_REVERSE_SUBTRACT_EXT () {0x800B}
sub GL_FUNC_SUBTRACT_EXT () {0x800A}
sub GL_GEQUAL () {0x0206}
sub GL_GREATER () {0x0204}
sub GL_GREEN () {0x1904}
sub GL_GREEN_BIAS () {0x0D19}
sub GL_GREEN_BITS () {0x0D53}
sub GL_GREEN_SCALE () {0x0D18}
sub GL_HINT_BIT () {0x00008000}
sub GL_INCR () {0x1E02}
sub GL_INDEX_ARRAY () {0x8077}
sub GL_INDEX_ARRAY_COUNT_EXT () {0x8087}
sub GL_INDEX_ARRAY_EXT () {0x8077}
sub GL_INDEX_ARRAY_POINTER () {0x8091}
sub GL_INDEX_ARRAY_POINTER_EXT () {0x8091}
sub GL_INDEX_ARRAY_STRIDE () {0x8086}
sub GL_INDEX_ARRAY_STRIDE_EXT () {0x8086}
sub GL_INDEX_ARRAY_TYPE () {0x8085}
sub GL_INDEX_ARRAY_TYPE_EXT () {0x8085}
sub GL_INDEX_BITS () {0x0D51}
sub GL_INDEX_CLEAR_VALUE () {0x0C20}
sub GL_INDEX_LOGIC_OP () {0x0BF1}
sub GL_INDEX_MODE () {0x0C30}
sub GL_INDEX_OFFSET () {0x0D13}
sub GL_INDEX_SHIFT () {0x0D12}
sub GL_INDEX_WRITEMASK () {0x0C21}
sub GL_INT () {0x1404}
sub GL_INTENSITY () {0x8049}
sub GL_INTENSITY12 () {0x804C}
sub GL_INTENSITY12_EXT () {0x804C}
sub GL_INTENSITY16 () {0x804D}
sub GL_INTENSITY16_EXT () {0x804D}
sub GL_INTENSITY4 () {0x804A}
sub GL_INTENSITY4_EXT () {0x804A}
sub GL_INTENSITY8 () {0x804B}
sub GL_INTENSITY8_EXT () {0x804B}
sub GL_INTENSITY_EXT () {0x8049}
sub GL_INVALID_ENUM () {0x0500}
sub GL_INVALID_OPERATION () {0x0502}
sub GL_INVALID_VALUE () {0x0501}
sub GL_INVERT () {0x150A}
sub GL_KEEP () {0x1E00}
sub GL_LEFT () {0x0406}
sub GL_LEQUAL () {0x0203}
sub GL_LESS () {0x0201}
sub GL_LIGHT0 () {0x4000}
sub GL_LIGHT1 () {0x4001}
sub GL_LIGHT2 () {0x4002}
sub GL_LIGHT3 () {0x4003}
sub GL_LIGHT4 () {0x4004}
sub GL_LIGHT5 () {0x4005}
sub GL_LIGHT6 () {0x4006}
sub GL_LIGHT7 () {0x4007}
sub GL_LIGHTING () {0x0B50}
sub GL_LIGHTING_BIT () {0x00000040}
sub GL_LIGHT_MODEL_AMBIENT () {0x0B53}
sub GL_LIGHT_MODEL_LOCAL_VIEWER () {0x0B51}
sub GL_LIGHT_MODEL_TWO_SIDE () {0x0B52}
sub GL_LINE () {0x1B01}
sub GL_LINEAR () {0x2601}
sub GL_LINEAR_ATTENUATION () {0x1208}
sub GL_LINEAR_MIPMAP_LINEAR () {0x2703}
sub GL_LINEAR_MIPMAP_NEAREST () {0x2701}
sub GL_LINES () {0x0001}
sub GL_LINE_BIT () {0x00000004}
sub GL_LINE_LOOP () {0x0002}
sub GL_LINE_RESET_TOKEN () {0x0707}
sub GL_LINE_SMOOTH () {0x0B20}
sub GL_LINE_SMOOTH_HINT () {0x0C52}
sub GL_LINE_STIPPLE () {0x0B24}
sub GL_LINE_STIPPLE_PATTERN () {0x0B25}
sub GL_LINE_STIPPLE_REPEAT () {0x0B26}
sub GL_LINE_STRIP () {0x0003}
sub GL_LINE_TOKEN () {0x0702}
sub GL_LINE_WIDTH () {0x0B21}
sub GL_LINE_WIDTH_GRANULARITY () {0x0B23}
sub GL_LINE_WIDTH_RANGE () {0x0B22}
sub GL_LIST_BASE () {0x0B32}
sub GL_LIST_BIT () {0x00020000}
sub GL_LIST_INDEX () {0x0B33}
sub GL_LIST_MODE () {0x0B30}
sub GL_LOAD () {0x0101}
sub GL_LOGIC_OP () {GL_INDEX_LOGIC_OP}
sub GL_LOGIC_OP_MODE () {0x0BF0}
sub GL_LUMINANCE () {0x1909}
sub GL_LUMINANCE12 () {0x8041}
sub GL_LUMINANCE12_ALPHA12 () {0x8047}
sub GL_LUMINANCE12_ALPHA12_EXT () {0x8047}
sub GL_LUMINANCE12_ALPHA4 () {0x8046}
sub GL_LUMINANCE12_ALPHA4_EXT () {0x8046}
sub GL_LUMINANCE12_EXT () {0x8041}
sub GL_LUMINANCE16 () {0x8042}
sub GL_LUMINANCE16_ALPHA16 () {0x8048}
sub GL_LUMINANCE16_ALPHA16_EXT () {0x8048}
sub GL_LUMINANCE16_EXT () {0x8042}
sub GL_LUMINANCE4 () {0x803F}
sub GL_LUMINANCE4_ALPHA4 () {0x8043}
sub GL_LUMINANCE4_ALPHA4_EXT () {0x8043}
sub GL_LUMINANCE4_EXT () {0x803F}
sub GL_LUMINANCE6_ALPHA2 () {0x8044}
sub GL_LUMINANCE6_ALPHA2_EXT () {0x8044}
sub GL_LUMINANCE8 () {0x8040}
sub GL_LUMINANCE8_ALPHA8 () {0x8045}
sub GL_LUMINANCE8_ALPHA8_EXT () {0x8045}
sub GL_LUMINANCE8_EXT () {0x8040}
sub GL_LUMINANCE_ALPHA () {0x190A}
sub GL_MAP1_COLOR_4 () {0x0D90}
sub GL_MAP1_GRID_DOMAIN () {0x0DD0}
sub GL_MAP1_GRID_SEGMENTS () {0x0DD1}
sub GL_MAP1_INDEX () {0x0D91}
sub GL_MAP1_NORMAL () {0x0D92}
sub GL_MAP1_TEXTURE_COORD_1 () {0x0D93}
sub GL_MAP1_TEXTURE_COORD_2 () {0x0D94}
sub GL_MAP1_TEXTURE_COORD_3 () {0x0D95}
sub GL_MAP1_TEXTURE_COORD_4 () {0x0D96}
sub GL_MAP1_VERTEX_3 () {0x0D97}
sub GL_MAP1_VERTEX_4 () {0x0D98}
sub GL_MAP2_COLOR_4 () {0x0DB0}
sub GL_MAP2_GRID_DOMAIN () {0x0DD2}
sub GL_MAP2_GRID_SEGMENTS () {0x0DD3}
sub GL_MAP2_INDEX () {0x0DB1}
sub GL_MAP2_NORMAL () {0x0DB2}
sub GL_MAP2_TEXTURE_COORD_1 () {0x0DB3}
sub GL_MAP2_TEXTURE_COORD_2 () {0x0DB4}
sub GL_MAP2_TEXTURE_COORD_3 () {0x0DB5}
sub GL_MAP2_TEXTURE_COORD_4 () {0x0DB6}
sub GL_MAP2_VERTEX_3 () {0x0DB7}
sub GL_MAP2_VERTEX_4 () {0x0DB8}
sub GL_MAP_COLOR () {0x0D10}
sub GL_MAP_STENCIL () {0x0D11}
sub GL_MATRIX_MODE () {0x0BA0}
sub GL_MAX_ATTRIB_STACK_DEPTH () {0x0D35}
sub GL_MAX_CLIENT_ATTRIB_STACK_DEPTH () {0x0D3B}
sub GL_MAX_CLIP_PLANES () {0x0D32}
sub GL_MAX_EVAL_ORDER () {0x0D30}
sub GL_MAX_EXT () {0x8008}
sub GL_MAX_LIGHTS () {0x0D31}
sub GL_MAX_LIST_NESTING () {0x0B31}
sub GL_MAX_MODELVIEW_STACK_DEPTH () {0x0D36}
sub GL_MAX_NAME_STACK_DEPTH () {0x0D37}
sub GL_MAX_PIXEL_MAP_TABLE () {0x0D34}
sub GL_MAX_PROJECTION_STACK_DEPTH () {0x0D38}
sub GL_MAX_TEXTURE_SIZE () {0x0D33}
sub GL_MAX_TEXTURE_STACK_DEPTH () {0x0D39}
sub GL_MAX_VIEWPORT_DIMS () {0x0D3A}
sub GL_MIN_EXT () {0x8007}
sub GL_MODELVIEW () {0x1700}
sub GL_MODELVIEW_MATRIX () {0x0BA6}
sub GL_MODELVIEW_STACK_DEPTH () {0x0BA3}
sub GL_MODULATE () {0x2100}
sub GL_MULT () {0x0103}
sub GL_N3F_V3F () {0x2A25}
sub GL_NAME_STACK_DEPTH () {0x0D70}
sub GL_NAND () {0x150E}
sub GL_NEAREST () {0x2600}
sub GL_NEAREST_MIPMAP_LINEAR () {0x2702}
sub GL_NEAREST_MIPMAP_NEAREST () {0x2700}
sub GL_NEVER () {0x0200}
sub GL_NICEST () {0x1102}
sub GL_NONE () {0}
sub GL_NOOP () {0x1505}
sub GL_NOR () {0x1508}
sub GL_NORMALIZE () {0x0BA1}
sub GL_NORMAL_ARRAY () {0x8075}
sub GL_NORMAL_ARRAY_COUNT_EXT () {0x8080}
sub GL_NORMAL_ARRAY_EXT () {0x8075}
sub GL_NORMAL_ARRAY_POINTER () {0x808F}
sub GL_NORMAL_ARRAY_POINTER_EXT () {0x808F}
sub GL_NORMAL_ARRAY_STRIDE () {0x807F}
sub GL_NORMAL_ARRAY_STRIDE_EXT () {0x807F}
sub GL_NORMAL_ARRAY_TYPE () {0x807E}
sub GL_NORMAL_ARRAY_TYPE_EXT () {0x807E}
sub GL_NOTEQUAL () {0x0205}
sub GL_NO_ERROR () {0}
sub GL_OBJECT_LINEAR () {0x2401}
sub GL_OBJECT_PLANE () {0x2501}
sub GL_ONE () {1}
sub GL_ONE_MINUS_CONSTANT_ALPHA_EXT () {0x8004}
sub GL_ONE_MINUS_CONSTANT_COLOR_EXT () {0x8002}
sub GL_ONE_MINUS_DST_ALPHA () {0x0305}
sub GL_ONE_MINUS_DST_COLOR () {0x0307}
sub GL_ONE_MINUS_SRC_ALPHA () {0x0303}
sub GL_ONE_MINUS_SRC_COLOR () {0x0301}
sub GL_OR () {0x1507}
sub GL_ORDER () {0x0A01}
sub GL_OR_INVERTED () {0x150D}
sub GL_OR_REVERSE () {0x150B}
sub GL_OUT_OF_MEMORY () {0x0505}
sub GL_PACK_ALIGNMENT () {0x0D05}
sub GL_PACK_LSB_FIRST () {0x0D01}
sub GL_PACK_ROW_LENGTH () {0x0D02}
sub GL_PACK_SKIP_PIXELS () {0x0D04}
sub GL_PACK_SKIP_ROWS () {0x0D03}
sub GL_PACK_SWAP_BYTES () {0x0D00}
sub GL_PASS_THROUGH_TOKEN () {0x0700}
sub GL_PERSPECTIVE_CORRECTION_HINT () {0x0C50}
sub GL_PIXEL_MAP_A_TO_A () {0x0C79}
sub GL_PIXEL_MAP_A_TO_A_SIZE () {0x0CB9}
sub GL_PIXEL_MAP_B_TO_B () {0x0C78}
sub GL_PIXEL_MAP_B_TO_B_SIZE () {0x0CB8}
sub GL_PIXEL_MAP_G_TO_G () {0x0C77}
sub GL_PIXEL_MAP_G_TO_G_SIZE () {0x0CB7}
sub GL_PIXEL_MAP_I_TO_A () {0x0C75}
sub GL_PIXEL_MAP_I_TO_A_SIZE () {0x0CB5}
sub GL_PIXEL_MAP_I_TO_B () {0x0C74}
sub GL_PIXEL_MAP_I_TO_B_SIZE () {0x0CB4}
sub GL_PIXEL_MAP_I_TO_G () {0x0C73}
sub GL_PIXEL_MAP_I_TO_G_SIZE () {0x0CB3}
sub GL_PIXEL_MAP_I_TO_I () {0x0C70}
sub GL_PIXEL_MAP_I_TO_I_SIZE () {0x0CB0}
sub GL_PIXEL_MAP_I_TO_R () {0x0C72}
sub GL_PIXEL_MAP_I_TO_R_SIZE () {0x0CB2}
sub GL_PIXEL_MAP_R_TO_R () {0x0C76}
sub GL_PIXEL_MAP_R_TO_R_SIZE () {0x0CB6}
sub GL_PIXEL_MAP_S_TO_S () {0x0C71}
sub GL_PIXEL_MAP_S_TO_S_SIZE () {0x0CB1}
sub GL_PIXEL_MODE_BIT () {0x00000020}
sub GL_POINT () {0x1B00}
sub GL_POINTS () {0x0000}
sub GL_POINT_BIT () {0x00000002}
sub GL_POINT_SIZE () {0x0B11}
sub GL_POINT_SIZE_GRANULARITY () {0x0B13}
sub GL_POINT_SIZE_RANGE () {0x0B12}
sub GL_POINT_SMOOTH () {0x0B10}
sub GL_POINT_SMOOTH_HINT () {0x0C51}
sub GL_POINT_TOKEN () {0x0701}
sub GL_POLYGON () {0x0009}
sub GL_POLYGON_BIT () {0x00000008}
sub GL_POLYGON_MODE () {0x0B40}
sub GL_POLYGON_OFFSET_BIAS_EXT () {0x8039}
sub GL_POLYGON_OFFSET_EXT () {0x8037}
sub GL_POLYGON_OFFSET_FACTOR () {0x8038}
sub GL_POLYGON_OFFSET_FACTOR_EXT () {0x8038}
sub GL_POLYGON_OFFSET_FILL () {0x8037}
sub GL_POLYGON_OFFSET_LINE () {0x2A02}
sub GL_POLYGON_OFFSET_POINT () {0x2A01}
sub GL_POLYGON_OFFSET_UNITS () {0x2A00}
sub GL_POLYGON_SMOOTH () {0x0B41}
sub GL_POLYGON_SMOOTH_HINT () {0x0C53}
sub GL_POLYGON_STIPPLE () {0x0B42}
sub GL_POLYGON_STIPPLE_BIT () {0x00000010}
sub GL_POLYGON_TOKEN () {0x0703}
sub GL_POSITION () {0x1203}
sub GL_PROJECTION () {0x1701}
sub GL_PROJECTION_MATRIX () {0x0BA7}
sub GL_PROJECTION_STACK_DEPTH () {0x0BA4}
sub GL_PROXY_TEXTURE_1D () {0x8063}
sub GL_PROXY_TEXTURE_1D_EXT () {0x8063}
sub GL_PROXY_TEXTURE_2D () {0x8064}
sub GL_PROXY_TEXTURE_2D_EXT () {0x8064}
sub GL_Q () {0x2003}
sub GL_QUADRATIC_ATTENUATION () {0x1209}
sub GL_QUADS () {0x0007}
sub GL_QUAD_STRIP () {0x0008}
sub GL_R () {0x2002}
sub GL_R3_G3_B2 () {0x2A10}
sub GL_READ_BUFFER () {0x0C02}
sub GL_RED () {0x1903}
sub GL_RED_BIAS () {0x0D15}
sub GL_RED_BITS () {0x0D52}
sub GL_RED_SCALE () {0x0D14}
sub GL_RENDER () {0x1C00}
sub GL_RENDERER () {0x1F01}
sub GL_RENDER_MODE () {0x0C40}
sub GL_REPEAT () {0x2901}
sub GL_REPLACE () {0x1E01}
sub GL_REPLACE_EXT () {0x8062}
sub GL_RETURN () {0x0102}
sub GL_RGB () {0x1907}
sub GL_RGB10 () {0x8052}
sub GL_RGB10_A2 () {0x8059}
sub GL_RGB10_A2_EXT () {0x8059}
sub GL_RGB10_EXT () {0x8052}
sub GL_RGB12 () {0x8053}
sub GL_RGB12_EXT () {0x8053}
sub GL_RGB16 () {0x8054}
sub GL_RGB16_EXT () {0x8054}
sub GL_RGB2_EXT () {0x804E}
sub GL_RGB4 () {0x804F}
sub GL_RGB4_EXT () {0x804F}
sub GL_RGB5 () {0x8050}
sub GL_RGB5_A1 () {0x8057}
sub GL_RGB5_A1_EXT () {0x8057}
sub GL_RGB5_EXT () {0x8050}
sub GL_RGB8 () {0x8051}
sub GL_RGB8_EXT () {0x8051}
sub GL_RGBA () {0x1908}
sub GL_RGBA12 () {0x805A}
sub GL_RGBA12_EXT () {0x805A}
sub GL_RGBA16 () {0x805B}
sub GL_RGBA16_EXT () {0x805B}
sub GL_RGBA2 () {0x8055}
sub GL_RGBA2_EXT () {0x8055}
sub GL_RGBA4 () {0x8056}
sub GL_RGBA4_EXT () {0x8056}
sub GL_RGBA8 () {0x8058}
sub GL_RGBA8_EXT () {0x8058}
sub GL_RGBA_MODE () {0x0C31}
sub GL_RIGHT () {0x0407}
sub GL_S () {0x2000}
sub GL_SCISSOR_BIT () {0x00080000}
sub GL_SCISSOR_BOX () {0x0C10}
sub GL_SCISSOR_TEST () {0x0C11}
sub GL_SELECT () {0x1C02}
sub GL_SELECTION_BUFFER_POINTER () {0x0DF3}
sub GL_SELECTION_BUFFER_SIZE () {0x0DF4}
sub GL_SET () {0x150F}
sub GL_SHADE_MODEL () {0x0B54}
sub GL_SHININESS () {0x1601}
sub GL_SHORT () {0x1402}
sub GL_SMOOTH () {0x1D01}
sub GL_SPECULAR () {0x1202}
sub GL_SPHERE_MAP () {0x2402}
sub GL_SPOT_CUTOFF () {0x1206}
sub GL_SPOT_DIRECTION () {0x1204}
sub GL_SPOT_EXPONENT () {0x1205}
sub GL_SRC_ALPHA () {0x0302}
sub GL_SRC_ALPHA_SATURATE () {0x0308}
sub GL_SRC_COLOR () {0x0300}
sub GL_STACK_OVERFLOW () {0x0503}
sub GL_STACK_UNDERFLOW () {0x0504}
sub GL_STENCIL () {0x1802}
sub GL_STENCIL_BITS () {0x0D57}
sub GL_STENCIL_BUFFER_BIT () {0x00000400}
sub GL_STENCIL_CLEAR_VALUE () {0x0B91}
sub GL_STENCIL_FAIL () {0x0B94}
sub GL_STENCIL_FUNC () {0x0B92}
sub GL_STENCIL_INDEX () {0x1901}
sub GL_STENCIL_PASS_DEPTH_FAIL () {0x0B95}
sub GL_STENCIL_PASS_DEPTH_PASS () {0x0B96}
sub GL_STENCIL_REF () {0x0B97}
sub GL_STENCIL_TEST () {0x0B90}
sub GL_STENCIL_VALUE_MASK () {0x0B93}
sub GL_STENCIL_WRITEMASK () {0x0B98}
sub GL_STEREO () {0x0C33}
sub GL_SUBPIXEL_BITS () {0x0D50}
sub GL_T () {0x2001}
sub GL_T2F_C3F_V3F () {0x2A2A}
sub GL_T2F_C4F_N3F_V3F () {0x2A2C}
sub GL_T2F_C4UB_V3F () {0x2A29}
sub GL_T2F_N3F_V3F () {0x2A2B}
sub GL_T2F_V3F () {0x2A27}
sub GL_T4F_C4F_N3F_V4F () {0x2A2D}
sub GL_T4F_V4F () {0x2A28}
sub GL_TEXTURE () {0x1702}
sub GL_TEXTURE_1D () {0x0DE0}
sub GL_TEXTURE_1D_BINDING_EXT () {0x8068}
sub GL_TEXTURE_2D () {0x0DE1}
sub GL_TEXTURE_2D_BINDING_EXT () {0x8069}
sub GL_TEXTURE_3D_BINDING_EXT () {0x806A}
sub GL_TEXTURE_ALPHA_SIZE () {0x805F}
sub GL_TEXTURE_ALPHA_SIZE_EXT () {0x805F}
sub GL_TEXTURE_BINDING_1D () {0x8068}
sub GL_TEXTURE_BINDING_2D () {0x8069}
sub GL_TEXTURE_BIT () {0x00040000}
sub GL_TEXTURE_BLUE_SIZE () {0x805E}
sub GL_TEXTURE_BLUE_SIZE_EXT () {0x805E}
sub GL_TEXTURE_BORDER () {0x1005}
sub GL_TEXTURE_BORDER_COLOR () {0x1004}
sub GL_TEXTURE_COMPONENTS () {GL_TEXTURE_INTERNAL_FORMAT}
sub GL_TEXTURE_COORD_ARRAY () {0x8078}
sub GL_TEXTURE_COORD_ARRAY_COUNT_EXT () {0x808B}
sub GL_TEXTURE_COORD_ARRAY_EXT () {0x8078}
sub GL_TEXTURE_COORD_ARRAY_POINTER () {0x8092}
sub GL_TEXTURE_COORD_ARRAY_POINTER_EXT () {0x8092}
sub GL_TEXTURE_COORD_ARRAY_SIZE () {0x8088}
sub GL_TEXTURE_COORD_ARRAY_SIZE_EXT () {0x8088}
sub GL_TEXTURE_COORD_ARRAY_STRIDE () {0x808A}
sub GL_TEXTURE_COORD_ARRAY_STRIDE_EXT () {0x808A}
sub GL_TEXTURE_COORD_ARRAY_TYPE () {0x8089}
sub GL_TEXTURE_COORD_ARRAY_TYPE_EXT () {0x8089}
sub GL_TEXTURE_ENV () {0x2300}
sub GL_TEXTURE_ENV_COLOR () {0x2201}
sub GL_TEXTURE_ENV_COMPONENT_ES () {108000}
sub GL_TEXTURE_ENV_MODE () {0x2200}
sub GL_TEXTURE_GEN_MODE () {0x2500}
sub GL_TEXTURE_GEN_Q () {0x0C63}
sub GL_TEXTURE_GEN_R () {0x0C62}
sub GL_TEXTURE_GEN_S () {0x0C60}
sub GL_TEXTURE_GEN_T () {0x0C61}
sub GL_TEXTURE_GREEN_SIZE () {0x805D}
sub GL_TEXTURE_GREEN_SIZE_EXT () {0x805D}
sub GL_TEXTURE_HEIGHT () {0x1001}
sub GL_TEXTURE_INTENSITY_SIZE () {0x8061}
sub GL_TEXTURE_INTENSITY_SIZE_EXT () {0x8061}
sub GL_TEXTURE_INTERNAL_FORMAT () {0x1003}
sub GL_TEXTURE_LUMINANCE_SIZE () {0x8060}
sub GL_TEXTURE_LUMINANCE_SIZE_EXT () {0x8060}
sub GL_TEXTURE_MAG_FILTER () {0x2800}
sub GL_TEXTURE_MATRIX () {0x0BA8}
sub GL_TEXTURE_MIN_FILTER () {0x2801}
sub GL_TEXTURE_PRIORITY () {0x8066}
sub GL_TEXTURE_PRIORITY_EXT () {0x8066}
sub GL_TEXTURE_RED_SIZE () {0x805C}
sub GL_TEXTURE_RED_SIZE_EXT () {0x805C}
sub GL_TEXTURE_RESIDENT () {0x8067}
sub GL_TEXTURE_RESIDENT_EXT () {0x8067}
sub GL_TEXTURE_STACK_DEPTH () {0x0BA5}
sub GL_TEXTURE_TOO_LARGE_EXT () {0x8065}
sub GL_TEXTURE_WIDTH () {0x1000}
sub GL_TEXTURE_WRAP_S () {0x2802}
sub GL_TEXTURE_WRAP_T () {0x2803}
sub GL_TRANSFORM_BIT () {0x00001000}
sub GL_TRIANGLES () {0x0004}
sub GL_TRIANGLE_FAN () {0x0006}
sub GL_TRIANGLE_STRIP () {0x0005}
sub GL_TRUE () {1}
sub GL_UNPACK_ALIGNMENT () {0x0CF5}
sub GL_UNPACK_LSB_FIRST () {0x0CF1}
sub GL_UNPACK_ROW_LENGTH () {0x0CF2}
sub GL_UNPACK_SKIP_PIXELS () {0x0CF4}
sub GL_UNPACK_SKIP_ROWS () {0x0CF3}
sub GL_UNPACK_SWAP_BYTES () {0x0CF0}
sub GL_UNSIGNED_BYTE () {0x1401}
sub GL_UNSIGNED_INT () {0x1405}
sub GL_UNSIGNED_SHORT () {0x1403}
sub GL_V2F () {0x2A20}
sub GL_V3F () {0x2A21}
sub GL_VENDOR () {0x1F00}
sub GL_VERSION () {0x1F02}
sub GL_VERSION_1_1 () {1}
sub GL_VERTEX_ARRAY () {0x8074}
sub GL_VERTEX_ARRAY_COUNT_EXT () {0x807D}
sub GL_VERTEX_ARRAY_EXT () {0x8074}
sub GL_VERTEX_ARRAY_POINTER () {0x808E}
sub GL_VERTEX_ARRAY_POINTER_EXT () {0x808E}
sub GL_VERTEX_ARRAY_SIZE () {0x807A}
sub GL_VERTEX_ARRAY_SIZE_EXT () {0x807A}
sub GL_VERTEX_ARRAY_STRIDE () {0x807C}
sub GL_VERTEX_ARRAY_STRIDE_EXT () {0x807C}
sub GL_VERTEX_ARRAY_TYPE () {0x807B}
sub GL_VERTEX_ARRAY_TYPE_EXT () {0x807B}
sub GL_VIEWPORT () {0x0BA2}
sub GL_VIEWPORT_BIT () {0x00000800}
sub GL_XOR () {0x1506}
sub GL_ZERO () {0}
sub GL_ZOOM_X () {0x0D16}
sub GL_ZOOM_Y () {0x0D17}
sub GrabFrozen () {4}
sub GrabInvalidTime () {2}
sub GrabModeAsync () {1}
sub GrabModeSync () {0}
sub GrabNotViewable () {3}
sub GrabSuccess () {0}
sub GraphicsExpose () {13}
sub GravityNotify () {24}
sub GrayScale () {1}
sub KeyPress () {2}
sub KeyPressMask () {(1<<0)}
sub KeyRelease () {3}
sub KeyReleaseMask () {(1<<1)}
sub KeymapNotify () {11}
sub KeymapStateMask () {(1<<14)}
sub LASTEvent () {35}
sub LeaveNotify () {8}
sub LeaveWindowMask () {(1<<5)}
sub LockMapIndex () {1}
sub LockMask () {(1<<1)}
sub MapNotify () {19}
sub MapRequest () {20}
sub MappingNotify () {34}
sub Mod1MapIndex () {3}
sub Mod1Mask () {(1<<3)}
sub Mod2MapIndex () {4}
sub Mod2Mask () {(1<<4)}
sub Mod3MapIndex () {5}
sub Mod3Mask () {(1<<5)}
sub Mod4MapIndex () {6}
sub Mod4Mask () {(1<<6)}
sub Mod5MapIndex () {7}
sub Mod5Mask () {(1<<7)}
sub MotionNotify () {6}
sub NoEventMask () {0}
sub NoExpose () {14}
sub NotifyAncestor () {0}
sub NotifyDetailNone () {7}
sub NotifyGrab () {1}
sub NotifyHint () {1}
sub NotifyInferior () {2}
sub NotifyNonlinear () {3}
sub NotifyNonlinearVirtual () {4}
sub NotifyNormal () {0}
sub NotifyPointer () {5}
sub NotifyPointerRoot () {6}
sub NotifyUngrab () {2}
sub NotifyVirtual () {1}
sub NotifyWhileGrabbed () {3}
sub OwnerGrabButtonMask () {(1<<24)}
sub PlaceOnBottom () {1}
sub PlaceOnTop () {0}
sub PointerMotionHintMask () {(1<<7)}
sub PointerMotionMask () {(1<<6)}
sub PropertyChangeMask () {(1<<22)}
sub PropertyDelete () {1}
sub PropertyNewValue () {0}
sub PropertyNotify () {28}
sub PseudoColor () {3}
sub ReparentNotify () {21}
sub ReplayKeyboard () {5}
sub ReplayPointer () {2}
sub ResizeRedirectMask () {(1<<18)}
sub ResizeRequest () {25}
sub SelectionClear () {29}
sub SelectionNotify () {31}
sub SelectionRequest () {30}
sub ShiftMapIndex () {0}
sub ShiftMask () {(1<<0)}
sub StaticColor () {2}
sub StaticGray () {0}
sub StructureNotifyMask () {(1<<17)}
sub SubstructureNotifyMask () {(1<<19)}
sub SubstructureRedirectMask () {(1<<20)}
sub SyncBoth () {7}
sub SyncKeyboard () {4}
sub SyncPointer () {1}
sub TrueColor () {4}
sub UnmapNotify () {18}
sub VisibilityChangeMask () {(1<<16)}
sub VisibilityFullyObscured () {2}
sub VisibilityNotify () {15}
sub VisibilityPartiallyObscured () {1}
sub VisibilityUnobscured () {0}
sub X_PROTOCOL () {11}
sub X_PROTOCOL_REVISION () {0}

%window_defaults=(
                'x'     => 0,
                'y'     => 0,
                'width' => 500,
                'height'=> 500,
                'parent'=> 0,
                'mask'  => StructureNotifyMask,
                'attributes'=> [GLX_RGBA],
        );
sub glpOpenWindow {
        # default values
        my(%a) = @_;
        my(%p) = %window_defaults;
        foreach $k (keys(%a)){
                defined($p{$k}) || warn "Not a valid parameter to glpOpenWindow: `$k'
\n";
                #print "parameter $k now ",$a{$k}," was ",$p{$k},"\n";
                $p{$k} = $a{$k};
        }
        glpcOpenWindow($p{x},$p{y},$p{width},$p{height},
                       $p{parent},$p{'mask'},
                       @{$p{attributes}});
}
1;

